#include "includes/carte.hpp"
#include <fcntl.h>
#include <iomanip>

K8055::K8055 (connexion &c, config &cf, bool simul)
{
  uh1=NULL;
  uh2=NULL;	
presente=simul;
co=c;    
cfg=cf;
card1_address=-1;
card2_address=-1;

if (presente==true) init();
else co.insert_log("mode simulation !");

}

void K8055::init()
{
  short r;
  usb_set_debug(DEBUG);
  usb_init();
  bool b1=false;
  bool b2=false;

string carte ="Carte K8055 trouve adresse ";

if (DEBUG==1) cout << "adresses: " << cfg.CARD1 << "  " << cfg.CARD2 << endl;

//firt card
if (cfg.CARD1 !=-1)
	{
	card1_address=cfg.CARD1;
	uh1= ueib_init(card1_address);
  	if(uh1 == NULL)
    		{
		card1_address=-1;
		presente=false;
      		cout << "Error connecting to first device" << endl;
      		// error type 1 in table error
      		insert_error(co,1);co.set_stop();co.close();exit(EXIT_FAILURE);
    		}
		else {
			std::ostringstream buffer;
			buffer << cfg.CARD1;
			co.insert_log(carte+buffer.str());
			b1=true;
		}
  	r=ueib_update(uh1);
  	if(r==-1) {co.insert_log("erreur carte prsente mais ne fonctionne pas");
		b1=false;} // can't read k8055 stopping
	}
//second card
if (cfg.CARD2 !=-1)
	{
	card2_address=cfg.CARD2;
	uh2= ueib_init(card2_address);
  	if(uh2 == NULL)
    		{
		card2_address=-1;
      		cout << "Error connecting to second device" << endl;
      		// error type 1 in table error
      		insert_error(co,1);co.set_stop();co.close();
    		}
		else {
			std::ostringstream buffer;
			buffer << cfg.CARD2;
		        co.insert_log(carte+buffer.str());
			b2=true;
		}
  	r=ueib_update(uh2);
  	if(r==-1) {co.insert_log("erreur carte prsente mais ne fonctionne pas");
		b2=false;} // can't read k8055 stopping
}
if ((b1==false) && (b2==false)) {
		co.insert_log("Pas de carte trouve");
		insert_error(co,1);
		co.set_stop();
		co.close();
		exit(EXIT_FAILURE);
		}

}

K8055::~K8055()
{
  if (presente==true)
    arret();
}

void K8055::arret()
{
  if (card1_address !=-1) ueib_close(uh1);
  if (card2_address !=-1) ueib_close(uh2);
}

void K8055::maj()
{
  if (presente==true)
    {
	if (card1_address!=-1)
	{
    		short r=ueib_update(uh1);
		if(r==-1) // can't read k8055
	  	{
	    	//error type 2 in table error
	    	insert_error(co,1);
	    	co.set_stop();co.close();
	    	exit(EXIT_FAILURE);
		}
	}	
	 if (card2_address!=-1)
	{
    		short r=ueib_update(uh2);
		if(r==-1) // can't read k8055
	  	{
	    	//error type 2 in table error
	    	insert_error(co,1);
	    	co.set_stop();co.close();
	    	//exit(EXIT_FAILURE);
		}
	} 

		//sleep(1);
	}
     
}

//mail and change state
void K8055::change_state_mail( short out,short card, short state,string s)
{
send_mail(s,co);
change_state(out,card,state,s);
}

//change output state - write log
short K8055::change_state( short out,short card, short state,string s)
{
string st,st2,s2;
time_t rawtime;
ueib_handle *uh=NULL;
bool p=presente;

if ( card==0 && card1_address==-1 && presente==true)
	{
	co.insert_log("erreur vous utilisez la carte d'adresse 00, elle n'est pas connecte !");
	presente=false;p=false;
	}
if (card==1 && card2_address==-1)
	{
	co.insert_log("erreur vous utilisez la carte d'adresse 01, elle n'est pas connecte !");
	p=false;
	}

if (card1_address==card) uh=uh1;
else if (card2_address==card) uh=uh2;

if (DEBUG==1) cout << "changing card out:"<<out<<" card "<<card << " state "<< state << endl;

time ( &rawtime );
s2=ctime(&rawtime);

st=st+ s2 + s ;
if (state==0) st2=" : off"; else st2=" : on";
st =st +st2;
if (p==true)
	{
	ueib_set_dig_output(uh,out,state);
	}  
co.insert_log(s+st2);
co.maj_out(card,out,state);
  
return 0;
}

//print status
void K8055::print_status()
{
/*
double temperature;

cout << "**** status ****\n";
cout << "=>inputs:\n";
if (ueib_get_dig_input(uh, cfg.STOP_PUMP)==1) cout  << setw(17) << "button 1:stopping pump for 10 min\n";
if (ueib_get_dig_input(uh, cfg.STOP_SKIMMER)==1) cout << setw(17) << "stopping skimmer\n";
if (ueib_get_dig_input(uh, cfg.OSMOLATION)==1) cout << setw(17) << "osmolation activated by captor\n";
if (ueib_get_dig_input(uh, cfg.RESERV)==1) cout << setw(17) << "RESERV EMPTY !!!\n";
temperature =(double) (101*ueib_get_ana_input(uh,INP_TEMP)/256)-23;
cout << "temperature : " << temperature << endl;	

cout << "\n=>outputs:\n";
if (ueib_get_dig_output(uh, cfg.OUTHQI)==1) cout << setw(17) << "HQI working\n"; else cout <<"HQI stopped\n";
if (ueib_get_dig_output(uh, cfg.OUTBLUE)==1) cout << setw(17) <<"Blue light working\n"; else cout <<"Blue light stopped\n";
if (ueib_get_dig_output(uh, cfg.OUTPUMP)==1) cout << setw(17) <<"Pump working\n"; else cout <<"Pump stopped\n";
if (ueib_get_dig_output(uh, cfg.OUTSKIMMER)==1) cout << setw(17) << "Skimmer working\n"; else cout <<"Skimmer stopped\n";
if (ueib_get_dig_output(uh, cfg.OUTOSMOLATION)==1) cout << setw(17) << "osmolation working\n"; else cout <<"Osmolation stopped\n";
if (ueib_get_dig_output(uh, cfg.OUTRAH)==1) cout << setw(17) << "RAH pump working\n"; else cout <<"RAH pump stopped\n";
if (ueib_get_dig_output(uh, cfg.OUTTEMP)==1) cout << setw(17) << "Chauffage working\n"; else cout <<"Chauffage stopped\n";
if (ueib_get_dig_output(uh, cfg.OUTPERI)==1) cout << setw(17) << "Peristatic pump working\n"; else cout <<"Peristatic pump stopped\n";
*/
}

short K8055::resetcounter(int nb,int card)
{
ueib_handle *uh=NULL;

if ( card==0 && card1_address==-1)
	{
	co.insert_log("erreur vous utilisez la carte d'adresse 00, elle n'est pas connecte !");
	return -1;
	}
if (card==1 && card2_address==-1)
	{
	co.insert_log("erreur vous utilisez la carte d'adresse 01, elle n'est pas connecte !");
	return -1;
	}

if (card1_address==card) uh=uh1;
else if (card2_address==card) uh=uh2;

if (presente==true)
ueib_reset_count_input(uh,nb);
maj();
return 0;
}

int K8055::readAnalog(int nb, int card)
{
ueib_handle *uh=NULL;
if (presente==true)
{
if ( card==0 && card1_address==-1)
	{
	co.insert_log("erreur vous utilisez la carte d'adresse 00, elle n'est pas connecte !");
	return 0;
	}
if (card==1 && card2_address==-1)
	{
	co.insert_log("erreur vous utilisez la carte d'adresse 01, elle n'est pas connecte !");
	return 0;
	}

if (card1_address==card) uh=uh1;
else if (card2_address==card) uh=uh2;

return ueib_get_ana_input(uh,nb); 
}
else return 0;
}


int K8055::readcounter(int nb,int card)
{
ueib_handle *uh=NULL;

if ( card==0 && card1_address==-1)
	{
	co.insert_log("erreur vous utilisez la carte d'adresse 00, elle n'est pas connecte !");
	return 0;
	}
if (card==1 && card2_address==-1)
	{
	co.insert_log("erreur vous utilisez la carte d'adresse 01, elle n'est pas connecte !");
	return 0;
	}

if (card1_address==card) uh=uh1;
else if (card2_address==card) uh=uh2;
if (presente==true) return ueib_get_count_input(uh,nb); else return 0;
}


int K8055::read(int bit,int card)
{
ueib_handle *uh=NULL;

if ( card==0 && card1_address==-1)
	{
	//co.insert_log("erreur vous utilisez la carte d'adresse 00, elle n'est pas connecte !");
	return 0;
	}
if (card==1 && card2_address==-1)
	{
	//co.insert_log("erreur vous utilisez la carte d'adresse 01, elle n'est pas connecte !");
	return 0;
	}

if (card1_address==card) uh=uh1;
else if (card2_address==card) uh=uh2;

	if (presente==true)
		return ueib_get_dig_input(uh,bit);
	else return 0;


}
