/*
  Name : effect.cpp
  Copyright : KPAMEGAN & ROUSSET
  Authors : KPAMEGAN Karel, ROUSSET Baptiste
  Description: file which contents function to make effects on a LCD screen
*/

#include "includes/effect.hpp"
#include <unistd.h>
#include <iostream>
#include <time.h>

using namespace std;

/***************************************FUNCTIONS**********************************************/

/*
  Name : nano
  Finality : deaden the process during a defined time
  Decription : deaden the process
*/
void nano(int speed)
{
  struct timespec req;
  req.tv_sec=0;
  if(speed==1)
    req.tv_nsec=500000000;
  if(speed==2)
  req.tv_nsec=100000000;
  nanosleep(&req,NULL);
  if(speed==3)
    sleep(1);
  if(speed==4)
    sleep(2);
}


/***************************************METHODS*************************************************/

/*
  Name : Effect
  Finality : this is a constructor
  Decription : empty ( call the Screen constructor )
*/
Effect::Effect()
{

}

/*
  Name : Effect
  Finality : this is a constructor
  Decription : allot the screen to the one which is given on argument
*/
Effect::Effect(Screen *screen)
{
  myScreen=screen;
}

/*
  Name : ~Effect
  Finality : this is the destructor
  Decription : empty
*/
Effect::~Effect()
{

}

/*
  Name : show_screen
  Finality : display the screen
  Decription : display the screen
*/
void Effect::show_screen(void)
{
  myScreen->show();
}

/*
  Name : blink_area
  Finality : blinking a defined area on the screen
  Decription : delete from #xs to #xe on the lines from #ys to #ye
               an rewite from #xs to #xe on the lines from #ys to #ye
               as such as #nb
*/
void Effect::blink_area(int xs,int xe,int ys,int ye,int nb)
{
  int size=xe-xs;
  int i=0,j=ys;
  while(nb>0)
    {
      myScreen->init_home_cursor();
      myScreen->go_down(ys);
      myScreen->go_right(xs);
      while(j<=ye)
      {
	  for(i=0;i<size;i++)
	    write(myScreen->fd," ",1);
	  myScreen->init_home_cursor();
	  myScreen->go_right(xs);
	  j++;
	  myScreen->go_down(j);
      }
      j=ys;
      nano(1);
      myScreen->show();
      nano(1);
      nb--;
    }
}

/*
  Name : blink_screen
  Finality : blinking the screen
  Decription : delete the screen and display it as such as wanted ( #nb )
*/
void Effect::blink_screen(int nb)
{
  while(nb>0)
    {
      nano(1);
      myScreen->delete_screen();
      nano(1);
      myScreen->show();
      nb--;
    }
}

/*
  Name : blink_line
  Finality : blinking a defined line
  Decription : delete the line given on argument and display it as such as wanted ( #nb )
*/
void Effect::blink_line(int nol,int nb)
{
  while(nb>0)
    {
      nano(1);
      myScreen->delete_line(nol);
      nano(1);
      myScreen->show();
      nb--;
    }
}

/*
  Name : disappear_screen_down
  Finality : disappear the screen from up to down
  Decription : delete line after line from the first to the last
*/
void Effect::disappear_screen_down(void)
{
  nano(2);
  myScreen->delete_line(0);
  nano(2);
  myScreen->delete_line(1);
  nano(2);
  myScreen->delete_line(2);
  nano(2);
  myScreen->delete_line(3);
  nano(2);
}

/*
  Name : disappear_screen_up
  Finality : disappear the screen from down to up
  Decription : delete line after line from the last to the first
*/
void Effect::disappear_screen_up(void)
{
  nano(2);
  myScreen->delete_line(3);
  nano(2);
  myScreen->delete_line(2);
  nano(2);
  myScreen->delete_line(1);
  nano(2);
  myScreen->delete_line(0);
  nano(2);
}

/*
  Name : pass_screen_down
  Finality : make the screen raveling on the down
  Decription : each line becomes the previous and the first becomes the last
               as such as it is given on argument
*/
void Effect::pass_screen_down(int nb)
{
  int i=0,j=0;
  char buf[20];
  char buf2[20];
  while(nb>0)
    {
      for(j=0;j<4;j++)
	{
	  for(i=0;i<20;i++)  
	    buf[i]=(myScreen->line0[i]);
	  for(i=0;i<20;i++) 
	    (myScreen->line0[i])=(myScreen->line3[i]);
	  for(i=0;i<20;i++) 
	    buf2[i]=(myScreen->line1[i]);
	  for(i=0;i<20;i++) 
	    (myScreen->line1[i])=buf[i];
	  for(i=0;i<20;i++) 
	    buf[i]=(myScreen->line2[i]);
	  for(i=0;i<20;i++) 
	    (myScreen->line2[i])=buf2[i];
	  for(i=0;i<20;i++) 
	    buf2[i]=(myScreen->line3[i]);
	  for(i=0;i<20;i++) 
	    (myScreen->line3[i])=buf[i];
	  nano(2);
	  myScreen->show();
	}
      nb--;
    }
}

/*
  Name : pass_screen_up
  Finality : make the screen raveling on the up
  Decription : each line becomes the next and the last becomes the first
               as such as it is given on argument
*/
void Effect::pass_screen_up(int nb)
{
  int i=0,j=0;
  char buf[20];
  char buf2[20];
  while(nb>0)
    {
      for(j=0;j<4;j++)
	{
	  for(i=0;i<20;i++)  
	    buf[i]=(myScreen->line3[i]);
	  for(i=0;i<20;i++) 
	    (myScreen->line3[i])=(myScreen->line0[i]);
	  for(i=0;i<20;i++) 
	    buf2[i]=(myScreen->line2[i]);
	  for(i=0;i<20;i++) 
	    (myScreen->line2[i])=buf[i];
	  for(i=0;i<20;i++) 
	    buf[i]=(myScreen->line1[i]);
	  for(i=0;i<20;i++) 
	    (myScreen->line1[i])=buf2[i];
	  for(i=0;i<20;i++) 
	    buf2[i]=(myScreen->line0[i]);
	  for(i=0;i<20;i++) 
	    (myScreen->line0[i])=buf[i];
	  myScreen->show();
	  nano(2);
	}
      nb--;
    }
}

/*
  Name : disappear_screen_right
  Finality : disappear the screen from left to right
  Decription : delete column after column from the first to the last
*/
void Effect::disappear_screen_right(void)
{
  int i=0;
  myScreen->init_home_cursor();
  for(i=0;i<20;i++)
    {
      write(myScreen->fd," ",1);
      myScreen->go_left(1);
      myScreen->go_down(1);
      write(myScreen->fd," ",1);
      myScreen->go_left(1);
      myScreen->go_down(1);
      write(myScreen->fd," ",1);
      myScreen->go_left(1);
      myScreen->go_down(1);
      write(myScreen->fd," ",1);
      myScreen->go_left(1);
      myScreen->go_right(1);
      myScreen->go_up(1);
      myScreen->go_up(1);
      myScreen->go_up(1);
      nano(2);
    }
}

/*
  Name : disappear_line_right
  Finality : disappear a line from left to right
  Decription : delete column after column from the first to the last
               of a line which is given on argument
*/
void Effect::disappear_line_right(int nol)
{
  int i=0;
  myScreen->set_cursor_line(nol);
  for(i=0;i<20;i++)
    {
      write(myScreen->fd," ",1);
      nano(2);
    }
}

/*
  Name : pass_screen_left
  Finality : make the screen raveling on the left
  Decription : each column becomes the next and the last becomes the first
               as such as it is given on argument
*/
void Effect::pass_screen_left(int nb)
{
  char s0,s1,s2,s3;
  int i=0,j=0;
  while(nb>0)
    {
      for(i=0;i<20;i++)
	{
	  s0=myScreen->line0[0];
	  s1=myScreen->line1[0];
	  s2=myScreen->line2[0];
	  s3=myScreen->line3[0];
	  for(j=0;j<19;j++)
	    {
	      myScreen->line0[j]=myScreen->line0[j+1];
	      myScreen->line1[j]=myScreen->line1[j+1];
	      myScreen->line2[j]=myScreen->line2[j+1];
	      myScreen->line3[j]=myScreen->line3[j+1];
	    }
	  myScreen->line0[19]=s0;
	  myScreen->line1[19]=s1;
	  myScreen->line2[19]=s2;
	  myScreen->line3[19]=s3;
	  nano(2);
	  myScreen->show();
  	}
      nb--;
    }
}

/*
  Name : pass_screen_right
  Finality : make the screen raveling on the right
  Decription : each column becomes the previous and the first becomes the last
               as such as it is given on argument
*/
void Effect::pass_screen_right(int nb)
{
  char s0,s1,s2,s3;
  int i=0,j=0;
  while(nb>0)
    {
      for(i=0;i<20;i++)
	{
	  s0=myScreen->line0[19];
	  s1=myScreen->line1[19];
	  s2=myScreen->line2[19];
	  s3=myScreen->line3[19];
	  for(j=19;j>0;j--)
	    {
	      myScreen->line0[j]=myScreen->line0[j-1];
	      myScreen->line1[j]=myScreen->line1[j-1];
	      myScreen->line2[j]=myScreen->line2[j-1];
	      myScreen->line3[j]=myScreen->line3[j-1];
	    }
	  myScreen->line0[0]=s0;
	  myScreen->line1[0]=s1;
	  myScreen->line2[0]=s2;
	  myScreen->line3[0]=s3;
	  nano(2);
	  myScreen->show();
  	}
      nb--;
    }
}

/*
  Name : pass_line_right
  Finality : make a line raveling on the right
  Decription : each column becomes the previous and the first becomes the last
               as such as it is given on argument
*/
void Effect::pass_line_right(int nol,int nb)
{
  int i=0,j=0;
  char s;
  char *p=NULL;
  switch(nol)
    {
    case 0 : p=myScreen->line0;break;
    case 1 : p=myScreen->line1;break;
    case 2 : p=myScreen->line2;break;
    case 3 : p=myScreen->line3;break;
    }
  while(nb>0)
    {
      for(i=0;i<20;i++)
	{
	  s=p[19];
	  for(j=19;j>0;j--)
	    {
	      p[j]=p[j-1];
	    }
	  p[0]=s;
	  nano(2);
	  myScreen->show();
	}
      nb--;
    }
}

/*
  Name : pass_line_left
  Finality : make a line raveling on the left
  Decription : each column becomes the next and the last becomes the first
               as such as it is given on argument
*/
void Effect::pass_line_left(int nol,int nb)
{
  int i=0,j=0;
  char s;
  char *p=NULL;
  switch(nol)
    {
    case 0 : p=myScreen->line0;break;
    case 1 : p=myScreen->line1;break;
    case 2 : p=myScreen->line2;break;
    case 3 : p=myScreen->line3;break;
    }
  while(nb>0)
    {
      for(i=0;i<20;i++)
	{
	  s=p[0];
	  for(j=0;j<19;j++)
	    {
	      p[j]=p[j+1];
	    }
	  p[19]=s;
	  nano(2);
	  myScreen->show();
	}
      nb--;
    }
}
