/*
  Name : automaton.hpp
  Copyright : MARTY & FOK
  Authors : MARTY Sbastien & FOK Pierre
  Description: file which contents functions of the automaton
*/

#ifndef _AUTOMATON_HPP_
#define _AUTOMATON_HPP_

#include "blue.hpp"
#include "carte.hpp"
#include "hqi.hpp"
#include "lampes.hpp"
#include "peristatic.hpp"
#include "rah.hpp"
#include "skimmer.hpp"
#include "temperature.hpp"
#include "osmolation.hpp"
#include "param.h"
#include "util.h"
#include "ventilator.hpp"
#include "phsonde.hpp"
#include "premont.hpp"
#include "rac.hpp"

#include <sstream>


class Automaton
{
private:
   //pointer on all modules
  HQI *hqi;
  Blue *blue;
  Blue *blue2;
  Peristatic *pump;
  Peristatic *pump2;
  Temperature *temperature;
  Skimmer *skimmer;
  Peristatic *peristatic [4];
  Rah *rah;
  Osmolation *osmolation;
  Ventilator *ventilator;
  phsonde *ph1;
  phsonde *ph2;	
  Premont *remont;
  Rac *rac;

public:

  //Attributes
  short getstart;	//state of the automaton
   time_t timer;
 
  HQI * getHqi(){return hqi;}
  Blue * getBlue(){return blue;}
  Blue * getBlue2(){return blue2;}
  Peristatic * getPump(){return pump;}
  Peristatic * getPump2(){return pump2;}
  Temperature * getTemperature(){return temperature;}
  Skimmer * getSkimmer(){return skimmer;}
  Peristatic * getPeristatic(int);
  Rah * getRah(){return rah;}
  Osmolation * getOsmolation(){return osmolation;}
  Ventilator * getVentilator(){return ventilator;}
  phsonde * getphsonde1(){return ph1;}
  phsonde * getphsonde2(){return ph2;}
  Premont * getPremont(){return remont;}
  Rac * getRac(){return rac;}

  //Methods
  Automaton(config &);
  ~Automaton();
 // void init();		//initialize the automaton and all its modules
  void restart (connexion &);	//restart the automaton

  void getInputState(K8055 &,config&);	//get the state all input modules
  


};

#endif
