#include "stdafx.h"
#include <stdlib.h>
#include "com_phidgets_Dictionary.h"
#include "../cphidgetdictionary.h"
#include "../csocket.h"
#include "phidget_jni.h"

static jfieldID dictionary_handle_fid;
static jclass dictionary_class;

void
com_phidgets_Dictionary_OnLoad(JNIEnv *env)
{
	if (!(dictionary_class = (*env)->FindClass(env, "com/phidgets/Dictionary")))
	{
		(*env)->ExceptionDescribe(env);
		(*env)->ExceptionClear(env);
		abort();
	}
	if (!(dictionary_class = (jclass)(*env)->NewGlobalRef(env, dictionary_class)))
	{
		(*env)->ExceptionDescribe(env);
		(*env)->ExceptionClear(env);
		abort();
	}
	if (!(dictionary_handle_fid = (*env)->GetFieldID(env, dictionary_class,
	  "handle", "J")))
	{
		(*env)->ExceptionDescribe(env);
		(*env)->ExceptionClear(env);
		abort();
	}
}

JNIEXPORT jboolean JNICALL
Java_com_phidgets_Dictionary_isAttached(JNIEnv *env, jobject obj)
{
	CPhidgetHandle h = (CPhidgetHandle)(uintptr_t)(*env)->GetLongField(env,
	    obj, dictionary_handle_fid);
	int error;
	int s;
	
	if ((error = CPhidget_getDeviceStatus(h, &s)))
		PH_THROW(error);

	return s == PHIDGET_ATTACHED;
}

JNIEXPORT jint JNICALL
Java_com_phidgets_Dictionary_getServerPort(JNIEnv *env, jobject obj)
{
	CPhidgetHandle h = (CPhidgetHandle)(uintptr_t)(*env)->GetLongField(env,
	    obj, dictionary_handle_fid);
	int error;
	int port;
	const char *addr;

	if ((error = CPhidget_getServerAddress(h, &addr, &port)))
		PH_THROW(error);

	return port;
}

JNIEXPORT jstring JNICALL
Java_com_phidgets_Dictionary_getServerAddress(JNIEnv *env, jobject obj)
{
	CPhidgetHandle h = (CPhidgetHandle)(uintptr_t)(*env)->GetLongField(env,
	    obj, dictionary_handle_fid);
	int error;
	int port;
	const char *addr;

	if ((error = CPhidget_getServerAddress(h, &addr, &port)))
		PH_THROW(error);

	return (*env)->NewStringUTF(env, addr);
}

JNIEXPORT jstring JNICALL
Java_com_phidgets_Dictionary_getServerID(JNIEnv *env, jobject obj)
{
	CPhidgetHandle h = (CPhidgetHandle)(uintptr_t)(*env)->GetLongField(env,
	    obj, dictionary_handle_fid);
	int error;
	const char *id;

	if ((error = CPhidget_getServerID(h, &id)))
		PH_THROW(error);

	return (*env)->NewStringUTF(env, id);
}

JNIEXPORT void JNICALL
Java_com_phidgets_Dictionary_nativeOpenRemote(JNIEnv *env, jobject obj, jstring serverID, jstring pass)
{
	int error;
    jboolean iscopy;
    const char *serverIDcopy = (*env)->GetStringUTFChars(
                env, serverID, &iscopy);
    const char *passcopy = (*env)->GetStringUTFChars(
                env, pass, &iscopy);

	CPhidgetDictionaryHandle h = (CPhidgetDictionaryHandle)(uintptr_t)
	    (*env)->GetLongField(env, obj, dictionary_handle_fid);

	if ((error = CPhidgetDictionary_openRemote(h, serverIDcopy, passcopy)))
		PH_THROW(error);
}

JNIEXPORT void JNICALL
Java_com_phidgets_Dictionary_nativeOpenRemoteIP(JNIEnv *env, jobject obj, jstring ipAddr, jint port, jstring pass)
{
	int error;
    jboolean iscopy;
    const char *ipAddrcopy = (*env)->GetStringUTFChars(
                env, ipAddr, &iscopy);
    const char *passcopy = (*env)->GetStringUTFChars(
                env, pass, &iscopy);

	CPhidgetDictionaryHandle h = (CPhidgetDictionaryHandle)(uintptr_t)
	    (*env)->GetLongField(env, obj, dictionary_handle_fid);

	if ((error = CPhidgetDictionary_openRemoteIP(h, ipAddrcopy, port, passcopy)))
		PH_THROW(error);
}

JNIEXPORT void JNICALL
Java_com_phidgets_Dictionary_nativeDelete(JNIEnv *env, jobject obj)
{
	int error;
	
	CPhidgetDictionaryHandle h = (CPhidgetDictionaryHandle)(uintptr_t)
	    (*env)->GetLongField(env, obj, dictionary_handle_fid);

	if ((error = CPhidgetDictionary_delete(h)))
		PH_THROW(error);
}

JNIEXPORT void JNICALL
Java_com_phidgets_Dictionary_nativeClose(JNIEnv *env, jobject obj)
{
	int error;
	
	CPhidgetDictionaryHandle h = (CPhidgetDictionaryHandle)(uintptr_t)
	    (*env)->GetLongField(env, obj, dictionary_handle_fid);

	if ((error = CPhidgetDictionary_close(h)))
		PH_THROW(error);
}

JNIEXPORT void JNICALL 
Java_com_phidgets_Dictionary_nativeAddKey(JNIEnv *env, jobject obj, jstring key, jstring val, jint persistent)
{
	int error;
	
    jboolean iscopy;
    const char *keyString = (*env)->GetStringUTFChars(
                env, key, &iscopy);
    const char *valString = (*env)->GetStringUTFChars(
                env, val, &iscopy);

	CPhidgetDictionaryHandle h = (CPhidgetDictionaryHandle)(uintptr_t)
	    (*env)->GetLongField(env, obj, dictionary_handle_fid);

	if ((error = CPhidgetDictionary_addKey(h, keyString, valString, persistent)))
		PH_THROW(error);
}

JNIEXPORT void JNICALL 
Java_com_phidgets_Dictionary_nativeRemoveKey(JNIEnv *env, jobject obj, jstring pattern)
{
	int error;
	
    jboolean iscopy;
    const char *patternString = (*env)->GetStringUTFChars(
                env, pattern, &iscopy);

	CPhidgetDictionaryHandle h = (CPhidgetDictionaryHandle)(uintptr_t)
	    (*env)->GetLongField(env, obj, dictionary_handle_fid);

	if ((error = CPhidgetDictionary_removeKey(h, patternString)))
		PH_THROW(error);
}

JNIEXPORT jstring JNICALL 
Java_com_phidgets_Dictionary_nativeGetKey(JNIEnv *env, jobject obj, jstring key)
{
	int error;

	char val[1024]; //TODO: unimpose this limit
	
    jboolean iscopy;
    const char *keyString = (*env)->GetStringUTFChars(
                env, key, &iscopy);

	CPhidgetDictionaryHandle h = (CPhidgetDictionaryHandle)(uintptr_t)
	    (*env)->GetLongField(env, obj, dictionary_handle_fid);

	if ((error = CPhidgetDictionary_getKey(h, keyString, val, sizeof(val))))
		PH_THROW(error);
	
	return (*env)->NewStringUTF(env, val);
}

JNIEXPORT jlong JNICALL
Java_com_phidgets_Dictionary_create(JNIEnv *env, jobject obj)
{
	int error;
	CPhidgetDictionaryHandle dict;
	
	if ((error = CPhidgetDictionary_create(&dict)))
		PH_THROW(error);
	return (jlong)(uintptr_t)dict; 
}
