#include "stdafx.h"
#include "cphidgetinterfacekit.h"
#include "cusb.h"
#include "csocket.h"
#include "cthread.h"

static int CCONV
CPhidgetInterfaceKit_makePacket(CPhidgetHandle, unsigned char *,
  unsigned int *);

int CCONV CPhidgetInterfaceKit_dataInput(CPhidgetHandle phid,
    unsigned char *buffer, int length);

int CCONV
CPhidgetInterfaceKit_initAfterOpen(CPhidgetHandle phid,
    unsigned char *buffer, int length)
{
	CPhidgetInterfaceKitHandle ifkit = (CPhidgetInterfaceKitHandle)phid;
	int j;

	if (!ifkit)
		return EPHIDGET_INVALIDARG;
		
	ifkit->phid.awdc_enabled = FALSE;
	ifkit->fullStateEcho = FALSE;

	if (ifkit->phid.deviceVersion >= 601 && ifkit->phid.deviceVersion < 700)
		ifkit->fullStateEcho = TRUE;
	if (ifkit->phid.deviceVersion >= 704 && ifkit->phid.deviceVersion < 800)
		ifkit->fullStateEcho = TRUE;
	if (ifkit->phid.deviceVersion >= 821 && ifkit->phid.deviceVersion < 900) {
		ifkit->phid.awdc_enabled = PTRUE;
		ifkit->fullStateEcho = TRUE;
	}
	
	/* textlcd888 */
	if (ifkit->phid.deviceVersion >= 120 && ifkit->phid.deviceVersion < 200) {
		ifkit->phid.awdc_enabled = PTRUE;
		ifkit->fullStateEcho = TRUE;
	}
	
	ifkit->ratiometric = PUNK;

	for (j = 0; j<ifkit->phid.attr.ifkit.numInputs; j++)
		ifkit->physicalState[j] = PUNK;
	for (j = 0; j<ifkit->phid.attr.ifkit.numSensors; j++) {
		ifkit->sensorChangeTrigger[j] = 10;
		ifkit->sensorRawValue[j] = 65535;
		ifkit->sensorValue[j] = 65535;
		ifkit->sensorLastValue[j] = 65535;
	}
	for (j = 0; j<ifkit->phid.attr.ifkit.numOutputs; j++) {
		ifkit->outputStates[j] = PUNK;
		ifkit->nextOutputStates[j] = PUNK;
		ifkit->outputEchoStates[j] = PUNK;
		ifkit->changedOutputs[j] = 0;
	}
	ifkit->lastChangedOutput = 0;

	if (ifkit->phid.deviceIDSpec==PHIDSPEC_ROTARY_TOUCH) {
		unsigned char buffer[8] = { 0 };
		int result;
		buffer[0] = 0x01;
		if ((result = CUSBSendPacket((CPhidgetHandle)ifkit, buffer)) != EPHIDGET_OK)
			return result;
	}

	if (ifkit->phid.deviceIDSpec==PHIDSPEC_LINEAR_TOUCH) {
		unsigned char buffer[8] = { 0 };
		int result;
		buffer[0] = 0x01;
		if ((result = CUSBSendPacket((CPhidgetHandle)ifkit, buffer)) != EPHIDGET_OK)
			return result;
		SLEEP(100);
		buffer[0] = 0x02;
		if ((result = CUSBSendPacket((CPhidgetHandle)ifkit, buffer)) != EPHIDGET_OK)
			return result;
	}

	return EPHIDGET_OK;
}

int CCONV
CPhidgetInterfaceKit_dataInput(CPhidgetHandle phid, unsigned char *buffer,
    int length)
{
	CPhidgetInterfaceKitHandle ifkit = (CPhidgetInterfaceKitHandle)phid;
	int normSensor, j, i, k;
	unsigned char inputValue;

	if (!ifkit)
		return EPHIDGET_INVALIDARG;

	if (ifkit->phid.networkInfo && 0) {
		/* Here we need to do device specific parsing... */
	} else {
		unsigned short newSensor[4];
		switch (ifkit->phid.deviceIDSpec) {
		case PHIDSPEC_INTERFACEKIT_4_8_8:
			newSensor[0] = ((unsigned char)buffer[3] +
			    ((unsigned char)buffer[2] & 0x0f) * 256);
			newSensor[1] = ((unsigned char)buffer[1] +
			    ((unsigned char)buffer[2] & 0xf0) * 16);
			newSensor[2] = ((unsigned char)buffer[6] +
			    ((unsigned char)buffer[5] & 0x0f) * 256);
			newSensor[3] = ((unsigned char)buffer[4] +
			    ((unsigned char)buffer[5] & 0xf0) * 16);

			for (i = 0; i<ifkit->phid.attr.ifkit.numSensors; i++) {
				ifkit->sensorRawValue[i] = newSensor[i];
				normSensor = (int)((double)newSensor[i] / 4.096);
				if (normSensor < 0)
					normSensor = 0;
				if (normSensor > 1000)
					normSensor = 1000;
				if (ifkit->sensorValue[i] == 65535) {
					ifkit->sensorLastValue[i] =
					    ifkit->sensorValue[i] = normSensor;
					IFIRE(ifkit, Sensor, i, normSensor);
				} else if (abs(ifkit->sensorLastValue[i] -
				    normSensor) >=
				    ifkit->sensorChangeTrigger[i]) {
					ifkit->sensorLastValue[i] =
					    ifkit->sensorValue[i] = normSensor;
					IFIRE(ifkit, Sensor, i, normSensor);
				}
			}
			for (i = 0, j = 128; i < 4; i++, j=j/2) {
				if ((buffer[0] & j) != 0) {
					if (ifkit->physicalState[i] !=
					    PFALSE) {
						ifkit->physicalState[i] =
						    PFALSE;
						IFIRE(ifkit, Input, i,
						    PFALSE);
					}
				} else {
					if (ifkit->physicalState[i] !=
					    PTRUE) {
						ifkit->physicalState[i] =
						    PTRUE;
						IFIRE(ifkit, Input, i, PTRUE);
					}
				}
			}
			for (i = 4, j = 1; i<8; i++, j*=2) {
				if ((buffer[0] & j) != 0) {
					if (ifkit->physicalState[i] !=
					    PFALSE) {
						ifkit->physicalState[i] =
						    PFALSE;
						IFIRE(ifkit, Input, i, PFALSE);
					}
				} else {
					if (ifkit->physicalState[i] !=
					    PTRUE) {
						ifkit->physicalState[i] =
						    PTRUE;
						IFIRE(ifkit, Input, i, PTRUE);
					}
				}
			}
			break;

		case PHIDSPEC_INTERFACEKIT_0_16_16:
			for (k = 0; k<2; k++)
				for (i = 0, j = 1; i<8; i++, j = j * 2)
				{
					if ((buffer[k] & j) == 0)
					{
						if (ifkit->physicalState[i+k*8] != PFALSE)
						{
							ifkit->physicalState[i+k*8] = PFALSE;
							IFIRE(ifkit, Input, i+k*8, PFALSE);
						}
					}
					else
					{
						if (ifkit->physicalState[i+k*8] != PTRUE)
						{
							ifkit->physicalState[i+k*8] = PTRUE;
							IFIRE(ifkit, Input, i+k*8, PTRUE);
						}
					}
				}
			if (ifkit->fullStateEcho) {
				if(ifkit->phid.deviceVersion == 601) //top 8 bits are flipped
				{
					for (i = 0, j = 1; i<8; i++,
						j = j * 2) {
						if ((buffer[2] & j) == 0) {
							if (ifkit->outputEchoStates[i] != PFALSE) {
								ifkit->outputEchoStates[i] = PFALSE;
								IFIRE(ifkit, Output, i, PFALSE);
							}
						} else {
							if (ifkit->outputEchoStates[i] != PTRUE) {
								ifkit->outputEchoStates[i] = PTRUE;
								IFIRE(ifkit, Output, i, PTRUE);
							}
						}
					}
					for (i = 7, j = 1; i>=0; i--,
						j = j * 2) {
						if ((buffer[3] & j) == 0) {
							if (ifkit->outputEchoStates[i+8] != PFALSE) {
								ifkit->outputEchoStates[i+8] = PFALSE;
								IFIRE(ifkit, Output, i+8, PFALSE);
							}
						} else {
							if (ifkit->outputEchoStates[i+8] != PTRUE) {
								ifkit->outputEchoStates[i+8] = PTRUE;
								IFIRE(ifkit, Output, i+8, PTRUE);
							}
						}
					}
				}
				else
				{
					for (k = 0; k<2; k++) {
						for (i = 0, j = 1; i<8; i++,
							j = j * 2) {
							if ((buffer[k+2] & j) == 0) {
								if (ifkit->outputEchoStates[i+k*8] != PFALSE) {
									ifkit->outputEchoStates[i+k*8] = PFALSE;
									IFIRE(ifkit, Output, i+k*8, PFALSE);
								}
							} else {
								if (ifkit->outputEchoStates[i+k*8] != PTRUE) {
									ifkit->outputEchoStates[i+k*8] = PTRUE;
									IFIRE(ifkit, Output, i+k*8, PTRUE);
								}
							}
						}
					}
				}
			}
			break;

		case PHIDSPEC_INTERFACEKIT_8_8_8:
		case PHIDSPEC_888_with_lcd:
			//printf("888 Data in datainput %2x %2x %2x %2x\n", buffer[0],buffer[1],buffer[2],buffer[3] );
			if (!(buffer[0] & 0x01)) {
				unsigned short newSensor[4];
				inputValue = buffer[1];
				newSensor[0] = ((unsigned char)buffer[2] + ((unsigned char)buffer[3] & 0x0f) * 256);
				newSensor[1] = ((unsigned char)buffer[4] + ((unsigned char)buffer[3] & 0xf0) * 16);
				newSensor[2] = ((unsigned char)buffer[5] + ((unsigned char)buffer[6] & 0x0f) * 256);
				newSensor[3] = ((unsigned char)buffer[7] + ((unsigned char)buffer[6] & 0xf0) * 16);
				for (i = 0; i < 4; i++) {
					ifkit->sensorRawValue[i] = newSensor[i];
					normSensor = (int)((double)newSensor[i] / 4.096);
					if (normSensor < 0) normSensor = 0;
					if (normSensor > 1000) normSensor = 1000;
					ifkit->sensorValue[i] = normSensor;
					//if (ifkit->sensorLastValue[i] == 0xFFFF) ifkit->sensorLastValue[i] = normSensor;
					//else if (abs(ifkit->sensorLastValue[i] - normSensor) >= ifkit->sensorChangeTrigger[i]) {
					if ((abs(ifkit->sensorLastValue[i] - normSensor) >= ifkit->sensorChangeTrigger[i])
						|| (ifkit->sensorLastValue[i] == 0xFFFF)) {
						ifkit->sensorLastValue[i] = normSensor;
						IFIRE(ifkit, Sensor, i, normSensor);
					}
				}




				for (i = 0, j = 1; i < ifkit->phid.attr.ifkit
				    .numInputs; i++, j=j<<1)
				{
					if (inputValue & j)
					{
						if (ifkit->physicalState[i] != PFALSE)
						{
							ifkit->physicalState[i] = PFALSE;
							IFIRE(ifkit, Input, i, PFALSE);
						}
					}
					else
					{
						if (ifkit->physicalState[i] != PTRUE)
						{
							ifkit->physicalState[i] = PTRUE;
							IFIRE(ifkit, Input, i, PTRUE);
						}
					}
				}
				if (ifkit->fullStateEcho)
				{
					for (i = 0, j = 0x10; i<4; i++, j = j * 2)
					{
						if ((buffer[0] & j) == 0)
						{
							CThread_mutex_lock(&ifkit->phid.outputLock);
							if (ifkit->outputStates[i] == PUNK)
								ifkit->outputStates[i] = PFALSE;
							CThread_mutex_unlock(&ifkit->phid.outputLock);
							if (ifkit->outputEchoStates[i] != PFALSE)
							{
								ifkit->outputEchoStates[i] = PFALSE;
								IFIRE(ifkit, Output, i, PFALSE);
							}
						}
						else
						{
							CThread_mutex_lock(&ifkit->phid.outputLock);
							if (ifkit->outputStates[i] == PUNK)
								ifkit->outputStates[i] = PTRUE;
							CThread_mutex_unlock(&ifkit->phid.outputLock);
							if (ifkit->outputEchoStates[i] != PTRUE)
							{
								ifkit->outputEchoStates[i] = PTRUE;
								IFIRE(ifkit, Output, i, PTRUE);
							}
						}
					}
				}


			}
			else
			{
				unsigned short newSensor[4];
				inputValue = buffer[1];
				newSensor[0] = ((unsigned char)buffer[2] + ((unsigned char)buffer[3] & 0x0f) * 256);
				newSensor[1] = ((unsigned char)buffer[4] + ((unsigned char)buffer[3] & 0xf0) * 16);
				newSensor[2] = ((unsigned char)buffer[5] + ((unsigned char)buffer[6] & 0x0f) * 256);
				newSensor[3] = ((unsigned char)buffer[7] + ((unsigned char)buffer[6] & 0xf0) * 16);
				for (i = 4; i < 8; i++)
				{
					ifkit->sensorRawValue[i] = newSensor[i - 4];
					normSensor = (int)((double)newSensor[i - 4] / 4.096);
					if (normSensor < 0) normSensor = 0;
					if (normSensor > 1000) normSensor = 1000;
					ifkit->sensorValue[i] = normSensor;
					//if (ifkit->sensorLastValue[i] == 0xFFFF) ifkit->sensorLastValue[i] = normSensor;
					//else if (abs(ifkit->sensorLastValue[i] - normSensor) >= ifkit->sensorChangeTrigger[i])
					if ((abs(ifkit->sensorLastValue[i] - normSensor) >= ifkit->sensorChangeTrigger[i])
						|| (ifkit->sensorLastValue[i] == 0xFFFF))
					{
						ifkit->sensorLastValue[i] = normSensor;
						IFIRE(ifkit, Sensor, i, normSensor);
					}
				}
				for (i = 0, j = 1; i<ifkit->phid.attr.ifkit
				    .numInputs; i++, j=j<<1)
				{
					if (inputValue & j)
					{
						if (ifkit->physicalState[i] != PFALSE)
						{
							ifkit->physicalState[i] = PFALSE;
							IFIRE(ifkit, Input, i, PFALSE);
						}
					}
					else
					{
						if (ifkit->physicalState[i] != PTRUE)
						{
							ifkit->physicalState[i] = PTRUE;
							IFIRE(ifkit, Input, i, PTRUE);
						}
					}
				}
				if (ifkit->fullStateEcho)
				{
					for (i = 4, j = 0x10; i<8; i++, j = j * 2)
					{
						if ((buffer[0] & j) == 0)
						{
							CThread_mutex_lock(&ifkit->phid.outputLock);
							if (ifkit->outputStates[i] == PUNK)
								ifkit->outputStates[i] = PFALSE;
							CThread_mutex_unlock(&ifkit->phid.outputLock);
							if (ifkit->outputEchoStates[i] != PFALSE)
							{
								ifkit->outputEchoStates[i] = PFALSE;
								IFIRE(ifkit, Output, i, PFALSE);
							}
						}
						else
						{
							CThread_mutex_lock(&ifkit->phid.outputLock);
							if (ifkit->outputStates[i] == PUNK)
								ifkit->outputStates[i] = PTRUE;
							CThread_mutex_unlock(&ifkit->phid.outputLock);
							if (ifkit->outputEchoStates[i] != PTRUE)
							{
								ifkit->outputEchoStates[i] = PTRUE;
								IFIRE(ifkit, Output, i, PTRUE);
							}
						}
					}
				}
			}

			break;

		case PHIDSPEC_INTERFACEKIT_0_5_7:
			for (j=1, i = 0; i<5; i++, j=j*2)
			{
				if ((buffer[0] & j) == 0)
				{
					if (ifkit->physicalState[i] != PFALSE)
					{
						ifkit->physicalState[i] = PFALSE;
						IFIRE(ifkit, Input, i, PFALSE);
					}
				}
				else
				{
					if (ifkit->physicalState[i] != PTRUE)
					{
						ifkit->physicalState[i] = PTRUE;
						IFIRE(ifkit, Input, i, PTRUE);
					}
				}
			}
			break;

		case PHIDSPEC_INTERFACEKIT_0_8_8:
			for (i = 0,j=1; i<8; i++,j*=2)
			{
				if ((buffer[0] & j) == 0)
				{
					if (ifkit->physicalState[i] != PFALSE)
					{
						ifkit->physicalState[i] = PFALSE;
						IFIRE(ifkit, Input, i, PFALSE);
					}
				}
				else
				{
					if (ifkit->physicalState[i] != PTRUE)
					{
						ifkit->physicalState[i] = PTRUE;
						IFIRE(ifkit, Input, i, PTRUE);
					}
				}
			}
			break;


		case PHIDSPEC_INTERFACEKIT_0_0_4:
			if (ifkit->fullStateEcho)
			{
				for (i = 0, j = 0x1; i<4; i++, j = j * 2)
				{
					if ((buffer[0] & j) == 0)
					{
						CThread_mutex_lock(&ifkit->phid.outputLock);
						if (ifkit->outputStates[i] == PUNK)
							ifkit->outputStates[i] = PFALSE;
						CThread_mutex_unlock(&ifkit->phid.outputLock);
						if (ifkit->outputEchoStates[i] != PFALSE)
						{
							ifkit->outputEchoStates[i] = PFALSE;
							IFIRE(ifkit, Output, i, PFALSE);
						}
					}
					else
					{
						CThread_mutex_lock(&ifkit->phid.outputLock);
						if (ifkit->outputStates[i] == PUNK)
							ifkit->outputStates[i] = PTRUE;
						CThread_mutex_unlock(&ifkit->phid.outputLock);
						if (ifkit->outputEchoStates[i] != PTRUE)
						{
							ifkit->outputEchoStates[i] = PTRUE;
							IFIRE(ifkit, Output, i, PTRUE);
						}
					}
				}
			}
			break;

		case PHIDSPEC_ROTARY_TOUCH:
		case PHIDSPEC_LINEAR_TOUCH:

			if (buffer[0] & 0x80)
			{
				if (ifkit->physicalState[0] != PTRUE)
				{
					ifkit->physicalState[0] = PTRUE;
					IFIRE(ifkit, Input, 0, PTRUE);
				}
				if (ifkit->physicalState[1] != PTRUE)
				{
					ifkit->physicalState[1] = PTRUE;
					IFIRE(ifkit, Input, 1, PTRUE);
				}

				ifkit->sensorRawValue[0] = buffer[0] & 0x7f;
				normSensor = (int)((double)ifkit->sensorRawValue[0] * 7.875);
				if (normSensor < 0) normSensor = 0;
				if (normSensor > 1000) normSensor = 1000;
				ifkit->sensorValue[0] = normSensor;
				if (abs(ifkit->sensorLastValue[0] - normSensor) >= ifkit->sensorChangeTrigger[0])
				{
					ifkit->sensorLastValue[0] = normSensor;
					IFIRE(ifkit, Sensor, 0, normSensor);
				}


			}
			else
			{
				if (ifkit->physicalState[0] != PFALSE)
				{
					ifkit->physicalState[0] = PFALSE;
					IFIRE(ifkit, Input, 0, PFALSE);
				}
				if (buffer[0] & 0x01)
				{
					if (ifkit->physicalState[1] != PTRUE)
					{
						ifkit->physicalState[1] = PTRUE;
						IFIRE(ifkit, Input, 1, PTRUE);
					}
				}
				else
				{
					if (ifkit->physicalState[1] != PFALSE)
					{
						ifkit->physicalState[1] = PFALSE;
						IFIRE(ifkit, Input, 1, PFALSE);
					}
				}
			}

			break;


		default:
			return EPHIDGET_UNEXPECTED;
		}

		return EPHIDGET_OK;

	}
	return EPHIDGET_NOTATTACHED;
}


int CCONV
CPhidgetInterfaceKit_create(CPhidgetInterfaceKitHandle *ifkitp)
{
	CPhidgetInterfaceKitHandle ifkit;

	if (!ifkitp)
		return EPHIDGET_INVALIDARG;

	if (!(ifkit = (CPhidgetInterfaceKitHandle)malloc(sizeof (*ifkit))))
		return EPHIDGET_NOMEMORY;
	ZEROMEM(ifkit, sizeof(*ifkit));

	ifkit->phid.deviceID = PHIDDEF_INTERFACEKIT;
	ifkit->phid.fptrInit = CPhidgetInterfaceKit_initAfterOpen;
	ifkit->phid.fptrData = CPhidgetInterfaceKit_dataInput;
	ifkit->phid.fptrMakePacket = CPhidgetInterfaceKit_makePacket;
	CThread_mutex_init(&ifkit->phid.lock);
	CThread_mutex_init(&ifkit->phid.writelock);
	CThread_mutex_init(&ifkit->phid.outputLock);
	CThread_create_event(&ifkit->phid.writeAvailableEvent);
	CThread_create_event(&ifkit->phid.writtenEvent);
	CPhidget_setStatus(&ifkit->phid, PHIDGET_NOTATTACHED);

	*ifkitp = ifkit;

	return EPHIDGET_OK;
}

int CCONV
CPhidgetInterfaceKit_getOutputState(CPhidgetInterfaceKitHandle ifkit,
    int Index, int *pVal)
{
	if (!ifkit)
		return EPHIDGET_INVALIDARG;
	if (!pVal)
		return EPHIDGET_INVALIDARG;
	*pVal = ifkit->outputEchoStates[Index];
	return EPHIDGET_OK;
}

int CCONV
CPhidgetInterfaceKit_setOutputState(CPhidgetInterfaceKitHandle ifkit,
  int i, int v)
{
	char key[100], val[10];
	if (!ifkit)
		return EPHIDGET_INVALIDARG;
	if (i >= ifkit->phid.attr.ifkit.numOutputs)
		return EPHIDGET_INVALIDARG;

	v = v ? PTRUE : PFALSE;

	if(ifkit->phid.networkInfo)
	{
		if(!CPhidgetSocketClient_isAttached(ifkit->phid.networkInfo))
			return EPHIDGET_NETWORK_NOTCONNECTED;
		snprintf(key, sizeof(key), "/PCK/%s/%d/Output/%d", ifkit->phid.deviceType, ifkit->phid.serialNumber, i);
		snprintf(val, sizeof(val), "%d", v);
		pdc_async_set(ifkit->phid.networkInfo->pdcs, key, val, PFALSE, internal_async_network_error_handler, &ifkit->phid);
	}
	else
	{

		CThread_mutex_lock(&ifkit->phid.writelock);
		ifkit->lastChangedOutput = i;

	again:
		if (!CPhidget_isAttached(&ifkit->phid))
		{
			CThread_mutex_unlock(&ifkit->phid.writelock);
			return EPHIDGET_NOTATTACHED;
		}
		CThread_mutex_lock(&ifkit->phid.outputLock);
		if (ifkit->changedOutputs[i]) {
			if (ifkit->nextOutputStates[i] != v) {
				CThread_mutex_unlock(&ifkit->phid.outputLock);
				CThread_wait_on_event(&ifkit->phid.writtenEvent, INFINITE);
				goto again;
			} else {
				CThread_mutex_unlock(&ifkit->phid.outputLock);
				CThread_mutex_unlock(&ifkit->phid.writelock);
				return EPHIDGET_OK;
			}
		} else {
			if (ifkit->outputStates[i] == v) {
				CThread_mutex_unlock(&ifkit->phid.outputLock);
				CThread_mutex_unlock(&ifkit->phid.writelock);
				return EPHIDGET_OK;
			}
			ifkit->changedOutputs[i] = 1;
			ifkit->nextOutputStates[i] = v;
			CThread_reset_event(&ifkit->phid.writtenEvent);
			CThread_mutex_unlock(&ifkit->phid.outputLock);
			CThread_set_event(&ifkit->phid.writeAvailableEvent);
		}

		if (!(ifkit->fullStateEcho)) {
			if (ifkit->outputEchoStates[i] != (unsigned char)v)
			{
				ifkit->outputEchoStates[i] = (unsigned char)v;
				IFIRE(ifkit, Output, i, v);
			}
		}

		CThread_mutex_unlock(&ifkit->phid.writelock);
	}
	return EPHIDGET_OK;
}

int CCONV
CPhidgetInterfaceKit_getSensorChangeTrigger(CPhidgetInterfaceKitHandle ifkit,
    int Index, int *pVal)
{
	if (!ifkit)
		return EPHIDGET_INVALIDARG;
	if (!pVal)
		return EPHIDGET_INVALIDARG;
	*pVal = ifkit->sensorChangeTrigger[Index];
	return EPHIDGET_OK;
}

int CCONV
CPhidgetInterfaceKit_setSensorChangeTrigger(CPhidgetInterfaceKitHandle ifkit,
    int Index, int newVal)
{
	char key[100], val[10];

	if (!ifkit)
		return EPHIDGET_INVALIDARG;

	if(ifkit->phid.networkInfo)
	{
		if(!CPhidgetSocketClient_isAttached(ifkit->phid.networkInfo))
			return EPHIDGET_NETWORK_NOTCONNECTED;
		snprintf(key, sizeof(key), "/PCK/%s/%ld/Trigger/%d", ifkit->phid.deviceType, ifkit->phid.serialNumber, Index);
		snprintf(val, sizeof(val), "%d", newVal);
		pdc_async_set(ifkit->phid.networkInfo->pdcs, key, val, PFALSE, internal_async_network_error_handler, &ifkit->phid);
	}

	ifkit->sensorChangeTrigger[Index] = newVal;
	return EPHIDGET_OK;
}

int CCONV
CPhidgetInterfaceKit_getNumInputs(CPhidgetInterfaceKitHandle ifkit,
    int *pVal)
{
	if (!ifkit)
		return EPHIDGET_INVALIDARG;
	if (!pVal)
		return EPHIDGET_INVALIDARG;
	*pVal = ifkit->phid.attr.ifkit.numInputs;
	return EPHIDGET_OK;
}

int CCONV
CPhidgetInterfaceKit_getNumOutputs(CPhidgetInterfaceKitHandle ifkit,
    int *pVal)
{
	if (!ifkit)
		return EPHIDGET_INVALIDARG;
	if (!pVal)
		return EPHIDGET_INVALIDARG;
	*pVal = ifkit->phid.attr.ifkit.numOutputs;
	return EPHIDGET_OK;
}

int CCONV
CPhidgetInterfaceKit_getNumSensors(CPhidgetInterfaceKitHandle ifkit,
    int *pVal)
{
	if (!ifkit)
		return EPHIDGET_INVALIDARG;
	if (!pVal)
		return EPHIDGET_INVALIDARG;
	*pVal = ifkit->phid.attr.ifkit.numSensors;
	return EPHIDGET_OK;
}

int CCONV
CPhidgetInterfaceKit_getInputState(CPhidgetInterfaceKitHandle ifkit,
    int Index, int *pVal)
{
	if (!ifkit)
		return EPHIDGET_INVALIDARG;
	if (!pVal)
		return EPHIDGET_INVALIDARG;
	*pVal = ifkit->physicalState[Index];
	return EPHIDGET_OK;
}

int CCONV
CPhidgetInterfaceKit_getSensorValue(CPhidgetInterfaceKitHandle ifkit,
    int Index, int *pVal)
{
	if (!ifkit)
		return EPHIDGET_INVALIDARG;
	if (!pVal)
		return EPHIDGET_INVALIDARG;
	*pVal = ifkit->sensorValue[Index];
	return EPHIDGET_OK;
}

int CCONV
CPhidgetInterfaceKit_getSensorRawValue(CPhidgetInterfaceKitHandle ifkit,
    int Index, int *pVal)
{
	if (!ifkit)
		return EPHIDGET_INVALIDARG;
	if (!pVal)
		return EPHIDGET_INVALIDARG;
	*pVal = ifkit->sensorRawValue[Index];
	return EPHIDGET_OK;
}

int CCONV
CPhidgetInterfaceKit_getRatiometric(CPhidgetInterfaceKitHandle ifkit,
    int *pVal)
{
	if (!ifkit)
		return EPHIDGET_INVALIDARG;
	if (!pVal)
		return EPHIDGET_INVALIDARG;
	*pVal = ifkit->ratiometric;
	return EPHIDGET_OK;
}

int CCONV
CPhidgetInterfaceKit_setRatiometric(CPhidgetInterfaceKitHandle ifkit,
    int newVal)
{
	unsigned char buffer[9];
	int k;
	char key[100], val[10];

	if (!ifkit)
		return EPHIDGET_INVALIDARG;

	if(ifkit->phid.networkInfo)
	{
		if(!CPhidgetSocketClient_isAttached(ifkit->phid.networkInfo))
			return EPHIDGET_NETWORK_NOTCONNECTED;
		snprintf(key, sizeof(key), "/PCK/%s/%d/Ratiometric", ifkit->phid.deviceType, ifkit->phid.serialNumber);
		snprintf(val, sizeof(val), "%d", newVal);
		pdc_async_set(ifkit->phid.networkInfo->pdcs, key, val, PFALSE, internal_async_network_error_handler, &ifkit->phid);
	}

	for (k = 0; k<9; k++)
		buffer[k] = 0;

	CThread_mutex_lock(&ifkit->phid.writelock);
	if (newVal)
		ifkit->ratiometric = 1;
	else
		ifkit->ratiometric = 0;

	if (CPhidget_isAttached(&ifkit->phid)) {
		switch(ifkit->phid.deviceIDSpec) {
		case PHIDSPEC_INTERFACEKIT_8_8_8:
		case PHIDSPEC_888_with_lcd:
			CThread_mutex_lock(&ifkit->phid.outputLock);
			CThread_reset_event(&ifkit->phid.writtenEvent);
			CThread_mutex_unlock(&ifkit->phid.outputLock);
			CThread_set_event(&ifkit->phid.writeAvailableEvent);
			CThread_mutex_unlock(&ifkit->phid.writelock);
			return EPHIDGET_OK;
		case PHIDSPEC_INTERFACEKIT_0_5_7:
		case PHIDSPEC_INTERFACEKIT_0_8_8:
		case PHIDSPEC_INTERFACEKIT_4_8_8:
		case PHIDSPEC_INTERFACEKIT_0_0_4:
		case PHIDSPEC_INTERFACEKIT_0_16_16:
			CThread_mutex_unlock(&ifkit->phid.writelock);
			return EPHIDGET_OK;
		}
	}
	CThread_mutex_unlock(&ifkit->phid.writelock);
	return EPHIDGET_UNEXPECTED;
}

int CCONV
CPhidgetInterfaceKit_set_OnInputChange_Handler(CPhidgetInterfaceKitHandle ifkit,
    int (CCONV *fptr)(CPhidgetInterfaceKitHandle, void *, int, int),
    void *userPtr)
{
	if (!ifkit)
		return EPHIDGET_INVALIDARG;

	ifkit->fptrInputChange = fptr;
	ifkit->fptrInputChangeptr = userPtr;

	return EPHIDGET_OK;
}

int CCONV
CPhidgetInterfaceKit_set_OnOutputChange_Handler(CPhidgetInterfaceKitHandle ifkit,
    int (CCONV *fptr)(CPhidgetInterfaceKitHandle, void *, int, int),
    void *userPtr)
{
	if (!ifkit)
		return EPHIDGET_INVALIDARG;

	ifkit->fptrOutputChange = fptr;
	ifkit->fptrOutputChangeptr = userPtr;

	return EPHIDGET_OK;
}

int CCONV
CPhidgetInterfaceKit_set_OnSensorChange_Handler(
    CPhidgetInterfaceKitHandle ifkit,
    int (CCONV *fptr)(CPhidgetInterfaceKitHandle, void *, int, int),
    void *userPtr)
{
	if (!ifkit)
		return EPHIDGET_INVALIDARG;

	ifkit->fptrSensorChange = fptr;
	ifkit->fptrSensorChangeptr = userPtr;

	return EPHIDGET_OK;
}

static int CCONV
CPhidgetInterfaceKit_makePacket(CPhidgetHandle phid, unsigned char *buffer,
    unsigned int *len)
{
	CPhidgetInterfaceKitHandle ifkit = (CPhidgetInterfaceKitHandle)phid;
	unsigned char ifkit488_lookup[8] = {0x00, 0x01, 0x02, 0x03, 0x07, 0x06,
	  0x05, 0x04};
	int i, j, k;

	if (!len || *len < 8)
		return EPHIDGET_INVALIDARG;
	if (!ifkit)
		return EPHIDGET_INVALIDARG;

	if (!CPhidget_isAttached(&ifkit->phid))
		return EPHIDGET_UNEXPECTED;

	CThread_mutex_lock(&ifkit->phid.outputLock);
	for (i = 0; i < 32; i++)
		if (ifkit->changedOutputs[i]) {
			ifkit->outputStates[i] = ifkit->nextOutputStates[i];
			ifkit->changedOutputs[i] = 0;
			ifkit->nextOutputStates[i] = PUNK;
		}

	switch(ifkit->phid.deviceIDSpec) {
	case PHIDSPEC_INTERFACEKIT_4_8_8:
		buffer[0] = ifkit488_lookup[ifkit->lastChangedOutput];
		if (ifkit->outputStates[ifkit->lastChangedOutput])
			buffer[0] |= 0x08;
		buffer[1] = 0;
		buffer[2] = 0;
		buffer[3] = 0;
		*len = 4;
		break;

	case PHIDSPEC_INTERFACEKIT_0_0_4:
		for (k = 0, j = 1; k<4; k++, j*=2) {
			if (ifkit->outputStates[k] == PUNK)
				ifkit->outputStates[k] = PFALSE;
			if (!ifkit->fullStateEcho && ifkit->outputEchoStates[k]
			  == PUNK)
				ifkit->outputEchoStates[k] =
				  ifkit->outputStates[k];
			if (ifkit->outputStates[k])
				buffer[0] |= j;
		}
		*len = 1;
		break;

	case PHIDSPEC_INTERFACEKIT_0_16_16:
		for (k = 0; k<2; k++)
			for (i = 0, j = 1; i<8; i++, j = j * 2) {
				if (ifkit->outputStates[k * 8 + i] == PUNK)
					ifkit->outputStates[k * 8 + i] = PFALSE;
				if (!ifkit->fullStateEcho &&
				  ifkit->outputEchoStates[k * 8 + i] == PUNK)
					ifkit->outputEchoStates[k * 8 + i] =
					  ifkit->outputStates[k * 8 + i];
				if (ifkit->outputStates[k * 8 + i])
					buffer[k] |= j;
			}
		*len = 2;
		break;

	case PHIDSPEC_INTERFACEKIT_8_8_8:
	case PHIDSPEC_888_with_lcd:
		for (k = 0, j = 1; k<8; k++, j *= 2) {
			if (ifkit->outputStates[k] == PUNK)
				ifkit->outputStates[k] = PFALSE;
			if (!ifkit->fullStateEcho && ifkit->outputEchoStates[k]
			  == PUNK)
				ifkit->outputEchoStates[k] =
				  ifkit->outputStates[k];
			if (ifkit->outputStates[k])
				buffer[0] |= j;
		}
		if(ifkit->ratiometric == PUNK)
		{
			ifkit->ratiometric = PTRUE;
		}
		buffer[3] = (char)ifkit->ratiometric;
		*len = 4;
		break;

	case PHIDSPEC_INTERFACEKIT_0_5_7:
		for (i = 0, j=1; i<8; i++, j*=2) {
			if (ifkit->outputStates[i] == PUNK)
				ifkit->outputStates[i] = PFALSE;
			if (!ifkit->fullStateEcho && ifkit->outputEchoStates[i]
			  == PUNK)
				ifkit->outputEchoStates[i] =
				  ifkit->outputStates[i];

			if (ifkit->outputStates[i])
				buffer[0] |= j;
		}
		buffer[7] = 0x10;  /* Signal an output */
		*len = 8;
		break;

	case PHIDSPEC_INTERFACEKIT_0_8_8:
		for (i = 0, j=1; i<8; i++, j*=2) {
			if (ifkit->outputStates[i] == PUNK)
				ifkit->outputStates[i] = PFALSE;
			if (!ifkit->fullStateEcho && ifkit->outputEchoStates[i]
			  == PUNK)
				ifkit->outputEchoStates[i] =
				  ifkit->outputStates[i];
			if (ifkit->outputStates[i])
				buffer[0] |= j;
		}
		buffer[7] = 0x10;  /* Signal an output */
		*len = 8;
		break;

	case PHIDSPEC_ROTARY_TOUCH:
	case PHIDSPEC_LINEAR_TOUCH:
		buffer[0] = ifkit->outputStates[ifkit->lastChangedOutput] ?
		  ifkit->lastChangedOutput : 0;
		*len = 1;
		break;
	}
	CThread_mutex_unlock(&ifkit->phid.outputLock);

	return EPHIDGET_OK;
}

