#include "stdafx.h"
#include "csocket.h"
#include "csocketevents.h"
#include "utils/utils.h"
#include "phidget21.h"
#include "math.h"

regex_t phidgetsetex;
regex_t managerex;


static int
hexval(unsigned char c)
{
	if (isdigit(c))
		return c - '0';
	c = tolower(c);
	if (c <= 'f')
		return c - 'a' + 10;
	return 0;
}

int phidget_accelerometer_set(CPhidgetHandle generic_phid, const char *setThing, int index, const char *state)
{
	int ret = EPHIDGET_OK;
	CPhidgetAccelerometerHandle phid = (CPhidgetAccelerometerHandle)generic_phid;
	if(!strncmp(setThing, "NumberOfAxes", sizeof("NumberOfAxes")))
	{
		int value = strtol(state, NULL, 10);
		phid->phid.attr.accelerometer.numAxis = value;
	}
	else if(!strncmp(setThing, "Acceleration", sizeof("Acceleration")))
	{
		double value = strtod(state, NULL);
		if(index < phid->phid.attr.accelerometer.numAxis?
			phid->phid.attr.accelerometer.numAxis:ACCEL_MAXAXES)
		{
			phid->axis[index] = value;

			if (fabs(value - phid->axisLastTrigger[index]) > phid->axisChangeTrigger[index])
			{
				FIRE(Acceleration) index, value);
				phid->axisLastTrigger[index] = value;
			}
		}
		else
			ret = EPHIDGET_OUTOFBOUNDS;
	}
	else if(!strncmp(setThing, "Trigger", sizeof("Trigger")))
	{
		double value = strtod(state, NULL);
		if(index < phid->phid.attr.accelerometer.numAxis?
			phid->phid.attr.accelerometer.numAxis:ACCEL_MAXAXES)
		{
			phid->axisChangeTrigger[index] = value;
		}
		else
			ret = EPHIDGET_OUTOFBOUNDS;
	}
	else{
		ret = EPHIDGET_INVALIDARG;
		DPRINT("Bad setType for Accelerometer: %s\n", setThing);
	}
	return ret;
}

int phidget_advancedservo_set(CPhidgetHandle generic_phid, const char *setThing, int index, const char *state)
{
	int ret = EPHIDGET_OK;
	CPhidgetAdvancedServoHandle phid = (CPhidgetAdvancedServoHandle)generic_phid;
	if(!strncmp(setThing, "NumberOfMotors", sizeof("NumberOfMotors")))
	{
		int value = strtol(state, NULL, 10);
		phid->phid.attr.advancedservo.numMotors = value;
	}
	else if(!strncmp(setThing, "Position", sizeof("Position")))
	{
		double value = strtod(state, NULL);
		if(index < phid->phid.attr.advancedservo.numMotors?
			phid->phid.attr.advancedservo.numMotors:ADVSERVO_MAXSERVOS)
		{
			if (phid->motorPWMCurrent[index] != value)
			{
				phid->motorPWMCurrent[index] = value;
				FIRE(Position) index, value);
			}
		}
		else
			ret = EPHIDGET_OUTOFBOUNDS;
	}
	else if(!strncmp(setThing, "Acceleration", sizeof("Acceleration")))
	{
		double value = strtod(state, NULL);
		if(index < phid->phid.attr.advancedservo.numMotors?
			phid->phid.attr.advancedservo.numMotors:ADVSERVO_MAXSERVOS)
		{
			phid->motorAcceleration[index] = value;
		}
		else
			ret = EPHIDGET_OUTOFBOUNDS;
	}
	else if(!strncmp(setThing, "Velocity", sizeof("Velocity")))
	{
		double value = strtod(state, NULL);
		if(index < phid->phid.attr.advancedservo.numMotors?
			phid->phid.attr.advancedservo.numMotors:ADVSERVO_MAXSERVOS)
		{
			phid->motorMaxVelocity[index] = value;
		}
		else
			ret = EPHIDGET_OUTOFBOUNDS;
	}
	else{
		ret = EPHIDGET_INVALIDARG;
		DPRINT("Bad setType for Accelerometer: %s\n", setThing);
	}
	return ret;
}

int phidget_encoder_set(CPhidgetHandle generic_phid, const char *setThing, int index, const char *state)
{
	int ret = EPHIDGET_OK;
	char *endPtr;
	CPhidgetEncoderHandle phid = (CPhidgetEncoderHandle)generic_phid;
	int value = strtol(state, &endPtr, 10);
	if(!strncmp(setThing, "NumberOfEncoders", sizeof("NumberOfEncoders")))
	{
		phid->phid.attr.encoder.numEncoders = value;
	}
	else if(!strncmp(setThing, "NumberOfInputs", sizeof("NumberOfInputs")))
	{
		phid->phid.attr.encoder.numInputs = value;
	}
	else if(!strncmp(setThing, "Input", sizeof("Input")))
	{
		if(index < phid->phid.attr.encoder.numInputs?
			phid->phid.attr.encoder.numInputs:ENCODER_MAXINPUTS)
		{
			if (phid->physicalState[index] != value)
			{
				phid->physicalState[index] = value;
				IFIRE(phid, Input, index, value);
			}
		}
		else
			ret = EPHIDGET_OUTOFBOUNDS;
	}
	else if(!strncmp(setThing, "ResetPosition", sizeof("ResetPosition")))
	{
		if(index < phid->phid.attr.encoder.numEncoders?
			phid->phid.attr.encoder.numEncoders:ENCODER_MAXENCODERS)
		{
			phid->encoderPosition[index] = value;
		}
		else
			ret = EPHIDGET_OUTOFBOUNDS;
	}
	else if(!strncmp(setThing, "Position", sizeof("Position")))
	{
		if(index < phid->phid.attr.encoder.numEncoders?
			phid->phid.attr.encoder.numEncoders:ENCODER_MAXENCODERS)
		{
			int tempC;
			int posnchange = strtol(endPtr+1, &endPtr, 10);
			int posn = strtol(endPtr+1, &endPtr, 10);
			
			phid->encoderPosition[index] = posn;
			
			tempC = (value - phid->encoderTimeStamp[index]);
			if (tempC > 30000) tempC = 30000;

			phid->encoderTimeStamp[index] = value;
			
			if (phid->fptrPositionChange)
				phid->fptrPositionChange(phid, phid->fptrPositionChangeptr, index, (unsigned short)tempC, posnchange);

		}
		else
			ret = EPHIDGET_OUTOFBOUNDS;
	}
	else{
		ret = EPHIDGET_INVALIDARG;
		DPRINT("Bad setType for Accelerometer: %s\n", setThing);
	}
	return ret;
}

int phidget_gps_set(CPhidgetHandle generic_phid, const char *setThing, int index, const char *state)
{
	int ret = EPHIDGET_OK;
	CPhidgetGPSHandle phid = (CPhidgetGPSHandle)generic_phid;
	if(!strncmp(setThing, "Data", sizeof("Data")))
	{
		//TODO: Implement this
		//parse_GPS_data(phid, state);
	}
	else if(!strncmp(setThing, "Trigger", sizeof("Trigger")))
	{
		double value = strtod(state, NULL);
		phid->PositionChangeTrigger = value * 0.0000899928055396;
	}
	else{
		ret = EPHIDGET_INVALIDARG;
		DPRINT("Bad setType for GPS: %s\n", setThing);
	}
	return ret;
}

int phidget_gyroscope_set(CPhidgetHandle generic_phid, const char *setThing, int index, const char *state)
{
	int ret = EPHIDGET_OK;
	CPhidgetGyroscopeHandle phid = (CPhidgetGyroscopeHandle)generic_phid;
	if(!strncmp(setThing, "NumberOfAxes", sizeof("NumberOfAxes")))
	{
		int value = strtol(state, NULL, 10);
		phid->phid.attr.gyroscope.numAxis = value;
	}
	else if(!strncmp(setThing, "AngularRate", sizeof("AngularRate")))
	{
		double value = strtod(state, NULL);
		if(index < phid->phid.attr.gyroscope.numAxis?
			phid->phid.attr.gyroscope.numAxis:GYRO_MAXAXES)
		{
			phid->axis[index] = value;

			if (fabs(value - phid->axisLastTrigger[index]) > phid->axisChangeTrigger[index])
			{
				FIRE(AngularRate) index, value);
				phid->axisLastTrigger[index] = value;
			}
		}
		else
			ret = EPHIDGET_OUTOFBOUNDS;
	}
	else if(!strncmp(setThing, "Trigger", sizeof("Trigger")))
	{
		double value = strtod(state, NULL);
		if(index < phid->phid.attr.gyroscope.numAxis?
			phid->phid.attr.gyroscope.numAxis:GYRO_MAXAXES)
		{
			phid->axisChangeTrigger[index] = value;
		}
		else
			ret = EPHIDGET_OUTOFBOUNDS;
	}
	else{
		ret = EPHIDGET_INVALIDARG;
		DPRINT("Bad setType for Accelerometer: %s\n", setThing);
	}
	return ret;
}

int phidget_interfacekit_set(CPhidgetHandle generic_phid, const char *setThing, int index, const char *state)
{
	int ret = EPHIDGET_OK;
	CPhidgetInterfaceKitHandle phid = (CPhidgetInterfaceKitHandle)generic_phid;
	int value = strtol(state, NULL, 10);
	if(!strncmp(setThing, "NumberOfSensors", sizeof("NumberOfSensors")))
	{
		phid->phid.attr.ifkit.numSensors = value;
	}
	else if(!strncmp(setThing, "NumberOfInputs", sizeof("NumberOfInputs")))
	{
		phid->phid.attr.ifkit.numInputs = value;
	}
	else if(!strncmp(setThing, "NumberOfOutputs", sizeof("NumberOfOutputs")))
	{
		phid->phid.attr.ifkit.numOutputs = value;
	}
	else if(!strncmp(setThing, "Input", sizeof("Input")))
	{
		if(index < phid->phid.attr.ifkit.numInputs?
			phid->phid.attr.ifkit.numInputs:IFKIT_MAXINPUTS)
		{
			if (phid->physicalState[index] != value)
			{
				phid->physicalState[index] = value;
				IFIRE(phid, Input, index, value);
			}
		}
		else
			ret = EPHIDGET_OUTOFBOUNDS;
	}
	else if(!strncmp(setThing, "Sensor", sizeof("Sensor")))
	{
		if(index < phid->phid.attr.ifkit.numSensors?
			phid->phid.attr.ifkit.numSensors:IFKIT_MAXSENSORS)
		{
			phid->sensorValue[index] = value;
			if (phid->sensorLastValue[index] == 0xFFFF) phid->sensorLastValue[index] = value;
			else if (abs(phid->sensorLastValue[index] - value) >= phid->sensorChangeTrigger[index]) {
				phid->sensorLastValue[index] = value;
				IFIRE(phid, Sensor, index, value);
			}
		}
		else
			ret = EPHIDGET_OUTOFBOUNDS;
	}
	else if(!strncmp(setThing, "Output", sizeof("Output")))
	{
		if(index < phid->phid.attr.ifkit.numOutputs?
			phid->phid.attr.ifkit.numOutputs:IFKIT_MAXOUTPUTS)
		{
			if (phid->outputStates[index] == PUNK)
				phid->outputStates[index] = value;
			if (phid->outputEchoStates[index] != value)
			{
				phid->outputEchoStates[index] = value;
				IFIRE(phid, Output, index, value);
			}
		}
		else
			ret = EPHIDGET_OUTOFBOUNDS;
	}
	else if(!strncmp(setThing, "Trigger", sizeof("Trigger")))
	{
		if(index < phid->phid.attr.ifkit.numSensors?
			phid->phid.attr.ifkit.numSensors:IFKIT_MAXSENSORS)
		{
			phid->sensorChangeTrigger[index] = value;
		}
		else
			ret = EPHIDGET_OUTOFBOUNDS;
	}
	else if(!strncmp(setThing, "Ratiometric", sizeof("Ratiometric")))
	{
		phid->ratiometric = value;
	}
	else{
		ret = EPHIDGET_INVALIDARG;
		DPRINT("Bad setType for InterfaceKit: %s\n", setThing);
	}
	return ret;
}

int phidget_led_set(CPhidgetHandle generic_phid, const char *setThing, int index, const char *state)
{
	int ret = EPHIDGET_OK;
	CPhidgetLEDHandle phid = (CPhidgetLEDHandle)generic_phid;
	int value = strtol(state, NULL, 10);
	if(!strncmp(setThing, "NumberOfLEDs", sizeof("NumberOfLEDs")))
	{
		phid->phid.attr.led.numLEDs = value;
	}
	else if(!strncmp(setThing, "Brightness", sizeof("Brightness")))
	{
		if(index < phid->phid.attr.led.numLEDs?
			phid->phid.attr.led.numLEDs:LED_MAXLEDS)
		{
			int i;
			i = (unsigned char)((((int)value) * 6400) / 10000); 
			if (i == 64) i = 63;
			phid->LED_Power[index] = i;
		}
		else
			ret = EPHIDGET_OUTOFBOUNDS;
	}
	else{
		ret = EPHIDGET_INVALIDARG;
		DPRINT("Bad setType for Accelerometer: %s\n", setThing);
	}
	return ret;
}

int phidget_motorcontrol_set(CPhidgetHandle generic_phid, const char *setThing, int index, const char *state)
{
	int ret = EPHIDGET_OK;
	CPhidgetMotorControlHandle phid = (CPhidgetMotorControlHandle)generic_phid;
	if(!strncmp(setThing, "NumberOfMotors", sizeof("NumberOfMotors")))
	{
		int value = strtol(state, NULL, 10);
		phid->phid.attr.motorcontrol.numMotors = value;
	}
	else if(!strncmp(setThing, "NumberOfInputs", sizeof("NumberOfInputs")))
	{
		int value = strtol(state, NULL, 10);
		phid->phid.attr.motorcontrol.numInputs = value;
	}
	else if(!strncmp(setThing, "Input", sizeof("Input")))
	{
		int value = strtol(state, NULL, 10);
		if(index < phid->phid.attr.motorcontrol.numInputs?
			phid->phid.attr.motorcontrol.numInputs:MOTORCONTROL_MAXINPUTS)
		{
			if (phid->inputStates[index] != value)
			{
				phid->inputStates[index] = value;
				FIRE(Input) index, value);
			}
		}
		else
			ret = EPHIDGET_OUTOFBOUNDS;
	}
	else if(!strncmp(setThing, "Velocity", sizeof("Velocity")))
	{
		double value = strtod(state, NULL);
		if(index < phid->phid.attr.motorcontrol.numMotors?
			phid->phid.attr.motorcontrol.numMotors:MOTORCONTROL_MAXMOTORS)
		{
			if (phid->motorPWMCurrent[index] != value)
			{
				phid->motorPWMCurrent[index] = value;
				FIRE(Motor) index, value);
			}
		}
		else
			ret = EPHIDGET_OUTOFBOUNDS;
	}
	else if(!strncmp(setThing, "Current", sizeof("Current")))
	{
		double value = strtod(state, NULL);
		if(index < phid->phid.attr.motorcontrol.numMotors?
			phid->phid.attr.motorcontrol.numMotors:MOTORCONTROL_MAXMOTORS)
		{
			if (phid->motorSensedCurrent[index] != value)
			{
				phid->motorSensedCurrent[index] = value;
				FIRE(Current) index, value);
			}
		}
		else
			ret = EPHIDGET_OUTOFBOUNDS;
	}
	else if(!strncmp(setThing, "Acceleration", sizeof("Acceleration")))
	{
		double value = strtod(state, NULL);
		if(index < phid->phid.attr.motorcontrol.numMotors?
			phid->phid.attr.motorcontrol.numMotors:MOTORCONTROL_MAXMOTORS)
		{
			phid->motorAcceleration[index] = value;
		}
		else
			ret = EPHIDGET_OUTOFBOUNDS;
	}
	else{
		ret = EPHIDGET_INVALIDARG;
		DPRINT("Bad setType for Accelerometer: %s\n", setThing);
	}
	return ret;
}

int phidget_phsensor_set(CPhidgetHandle generic_phid, const char *setThing, int index, const char *state)
{
	int ret = EPHIDGET_OK;
	CPhidgetPHSensorHandle phid = (CPhidgetPHSensorHandle)generic_phid;
	double value = strtod(state, NULL);
	if(!strncmp(setThing, "PH", sizeof("PH")))
	{
		phid->PH = value;
		if (fabs(phid->PH - phid->lastPH) >= phid->PHChangeTrigger)
		{
			phid->lastPH = phid->PH;
			if (phid->fptrPHChange)
			{
				phid->fptrPHChange(phid, phid->fptrPHChangeptr, phid->lastPH);
			}
		}
	}
	else if(!strncmp(setThing, "Trigger", sizeof("Trigger")))
	{
		phid->PHChangeTrigger = value;
	}
	else if(!strncmp(setThing, "Potential", sizeof("Potential")))
	{
		phid->Potential = value;
	}
	else{
		ret = EPHIDGET_INVALIDARG;
		DPRINT("Bad setType for PHSensor: %s\n", setThing);
	}
	return ret;
}

int phidget_rfid_set(CPhidgetHandle generic_phid, const char *setThing, int index, const char *state)
{
	int ret = EPHIDGET_OK;
	CPhidgetRFIDHandle phid = (CPhidgetRFIDHandle)generic_phid;
	int value = strtol(state, NULL, 10);
	if(!strncmp(setThing, "NumberOfOutputs", sizeof("NumberOfOutputs")))
	{
		phid->phid.attr.rfid.numOutputs = value;
	}
	else if(!strncmp(setThing, "Tag", sizeof("Tag")))
	{
		phid->lastTag[0] = (hexval(state[0])<<4)|hexval(state[1]);
		phid->lastTag[1] = (hexval(state[2])<<4)|hexval(state[3]);
		phid->lastTag[2] = (hexval(state[4])<<4)|hexval(state[5]);
		phid->lastTag[3] = (hexval(state[6])<<4)|hexval(state[7]);
		phid->lastTag[4] = (hexval(state[8])<<4)|hexval(state[9]);
		phid->lastTag[5] = 0;

		if (phid->fptrTag)
			phid->fptrTag(phid, phid->fptrTagptr, phid->lastTag);
	}
	else if(!strncmp(setThing, "TagLoss", sizeof("TagLoss")))
	{			
		if (phid->fptrTagLost)
			phid->fptrTagLost(phid, phid->fptrTagLostptr, phid->lastTag);
	}
	else if(!strncmp(setThing, "Output", sizeof("Output")))
	{
		if(index < phid->phid.attr.rfid.numOutputs?
			phid->phid.attr.rfid.numOutputs:RFID_MAXOUTPUTS)
		{
			if (value == 0)
			{
				if (phid->outputStates[index] == PUNK)
					phid->outputStates[index] = PFALSE;
				if (phid->outputEchoStates[index] != PFALSE)
				{
					phid->outputEchoStates[index] = PFALSE;
					FIRE(Output) index, PFALSE);
				}
			}
			else
			{
				if (phid->outputStates[index] == PUNK)
					phid->outputStates[index] = PTRUE;
				if (phid->outputEchoStates[index] != PTRUE)
				{	
					phid->outputEchoStates[index] = PTRUE;
					FIRE(Output) index, PTRUE);
				}
			}	
		}
		else
			ret = EPHIDGET_OUTOFBOUNDS;
	}
	else{
		ret = EPHIDGET_INVALIDARG;
		DPRINT("Bad setType for RFID: %s\n", setThing);
	}
	return ret;
}

int phidget_servo_set(CPhidgetHandle generic_phid, const char *setThing, int index, const char *state)
{
	int ret = EPHIDGET_OK;
	CPhidgetServoHandle phid = (CPhidgetServoHandle)generic_phid;
	if(!strncmp(setThing, "NumberOfMotors", sizeof("NumberOfMotors")))
	{
		int value = strtol(state, NULL, 10);
		phid->phid.attr.led.numLEDs = value;
	}
	else if(!strncmp(setThing, "Position", sizeof("Position")))
	{
		double value = strtod(state, NULL);
		int value2 = (int)(value * 10.6 + 243.8);
		if(index < phid->phid.attr.servo.numMotors?
			phid->phid.attr.servo.numMotors:SERVO_MAXSERVOS)
		{
			if (phid->microsecond_pulse[index] == 65535)
				phid->microsecond_pulse[index] = value2;
				
			if (value2 != phid->microsecond_Echo[index])
			{
				phid->microsecond_Echo[index]= value2;
				FIRE(MotorPosition) index, value);
			}
			phid->microsecond_Echo[index] = value2;
		}
		else
			ret = EPHIDGET_OUTOFBOUNDS;
	}
	else{
		ret = EPHIDGET_INVALIDARG;
		DPRINT("Bad setType for Accelerometer: %s\n", setThing);
	}
	return ret;
}

int phidget_stepper_set(CPhidgetHandle generic_phid, const char *setThing, int index, const char *state)
{
	int ret = EPHIDGET_OK;
	CPhidgetStepperHandle phid = (CPhidgetStepperHandle)generic_phid;
	if(!strncmp(setThing, "NumberOfMotors", sizeof("NumberOfMotors")))
	{
		int value = strtol(state, NULL, 10);
		phid->phid.attr.stepper.numMotors = value;
	}
	else if(!strncmp(setThing, "NumberOfInputs", sizeof("NumberOfInputs")))
	{
		int value = strtol(state, NULL, 10);
		phid->phid.attr.stepper.numInputs = value;
	}
	else if(!strncmp(setThing, "Input", sizeof("Input")))
	{
		//int value = strtol(state, NULL, 10);
		if(index < phid->phid.attr.stepper.numInputs?
			phid->phid.attr.stepper.numInputs:STEPPER_MAXINPUTS)
		{
			//TODO: Set something here
		}
		else
			ret = EPHIDGET_OUTOFBOUNDS;
	}
	else if(!strncmp(setThing, "Position", sizeof("Position")))
	{
#if defined(WINCE)
		long long value = (long long)strtol(state, NULL, 10);
#elif defined(_WINDOWS)
		long long value = (long long)_strtoi64(state, NULL, 10);
#else
		long long value = strtoll(state, NULL, 10);
#endif
		if(index < phid->phid.attr.stepper.numMotors?
			phid->phid.attr.stepper.numMotors:STEPPER_MAXSTEPPERS)
		{
			if (phid->motorPositionCurrent[index] != value)
			{
				phid->motorPositionCurrent[index] = value;
				FIRE(Position) index, value);
			}
		}
		else
			ret = EPHIDGET_OUTOFBOUNDS;
	}
	else if(!strncmp(setThing, "Acceleration", sizeof("Acceleration")))
	{
		double value = strtod(state, NULL);
		if(index < phid->phid.attr.stepper.numMotors?
			phid->phid.attr.stepper.numMotors:STEPPER_MAXSTEPPERS)
		{
			phid->motorAcceleration[index] = value;
		}
		else
			ret = EPHIDGET_OUTOFBOUNDS;
	}
	else if(!strncmp(setThing, "Velocity", sizeof("Velocity")))
	{
		double value = strtod(state, NULL);
		if(index < phid->phid.attr.stepper.numMotors?
			phid->phid.attr.stepper.numMotors:STEPPER_MAXSTEPPERS)
		{
			if (phid->motorSpeedCurrent[index] != value)
			{
				phid->motorSpeedCurrent[index] = value;
				FIRE(Speed) index, value);
			}
		}
		else
			ret = EPHIDGET_OUTOFBOUNDS;
	}
	else{
		ret = EPHIDGET_INVALIDARG;
		DPRINT("Bad setType for Stepper: %s\n", setThing);
	}
	return ret;
}

int phidget_temperaturesensor_set(CPhidgetHandle generic_phid, const char *setThing, int index, const char *state)
{
	int ret = EPHIDGET_OK;
	CPhidgetTemperatureSensorHandle phid = (CPhidgetTemperatureSensorHandle)generic_phid;
	if(!strncmp(setThing, "NumberOfSensors", sizeof("NumberOfSensors")))
	{
		int value = strtol(state, NULL, 10);
		phid->phid.attr.temperaturesensor.numTempInputs = value;
	}
	else if(!strncmp(setThing, "Temperature", sizeof("Temperature")))
	{
		double value = strtod(state, NULL);
		if(index < phid->phid.attr.temperaturesensor.numTempInputs?
			phid->phid.attr.temperaturesensor.numTempInputs:TEMPSENSOR_MAXSENSORS)
		{
			phid->Temperature[index] = value;
			if (fabs(phid->Temperature[index] - phid->lastTrigger[index]) >= phid->TempChangeTrigger[index])
			{
				phid->lastTrigger[index] = phid->Temperature[index];
				if (phid->fptrTemperatureChange)
				{
					phid->fptrTemperatureChange(phid, phid->fptrTemperatureChangeptr, index, phid->lastTrigger[index]);
				}
			}	
		}
		else
			ret = EPHIDGET_OUTOFBOUNDS;
	}
	else if(!strncmp(setThing, "Trigger", sizeof("Trigger")))
	{
		double value = strtod(state, NULL);
		if(index < phid->phid.attr.temperaturesensor.numTempInputs?
			phid->phid.attr.temperaturesensor.numTempInputs:TEMPSENSOR_MAXSENSORS)
		{
			phid->TempChangeTrigger[index] = value;
		}
		else
			ret = EPHIDGET_OUTOFBOUNDS;
	}
	else{
		ret = EPHIDGET_INVALIDARG;
		DPRINT("Bad setType for TemperatureSensor: %s\n", setThing);
	}
	return ret;
}

int phidget_textlcd_set(CPhidgetHandle generic_phid, const char *setThing, int index, const char *state)
{
	int ret = EPHIDGET_OK;
	CPhidgetTextLCDHandle phid = (CPhidgetTextLCDHandle)generic_phid;
	int value = strtol(state, NULL, 10);
	if(!strncmp(setThing, "NumberOfRows", sizeof("NumberOfRows")))
	{
		phid->phid.attr.textlcd.numRows = value;
	}
	else if(!strncmp(setThing, "NumberOfColumns", sizeof("NumberOfColumns")))
	{
		phid->phid.attr.textlcd.numColumns = value;
	}
	else if(!strncmp(setThing, "Backlight", sizeof("Backlight")))
	{
		phid->m_blnBacklight = (unsigned char)value;
	}
	else if(!strncmp(setThing, "CursorOn", sizeof("CursorOn")))
	{
		phid->m_blnCursorOn = (unsigned char)value;
	}
	else if(!strncmp(setThing, "CursorBlink", sizeof("CursorBlink")))
	{
		phid->m_blnCursorBlink = (unsigned char)value;
	}
	else if(!strncmp(setThing, "Contrast", sizeof("Contrast")))
	{
		phid->m_iContrast = (unsigned char)value;
	}
	else{
		ret = EPHIDGET_INVALIDARG;
		DPRINT("Bad setType for TextLCD: %s\n", setThing);
	}
	return ret;
}

int phidget_textled_set(CPhidgetHandle generic_phid, const char *setThing, int index, const char *state)
{
	int ret = EPHIDGET_OK;
	CPhidgetTextLEDHandle phid = (CPhidgetTextLEDHandle)generic_phid;
	int value = strtol(state, NULL, 10);
	if(!strncmp(setThing, "NumberOfRows", sizeof("NumberOfRows")))
	{
		phid->phid.attr.textled.numRows = value;
	}
	else if(!strncmp(setThing, "NumberOfColumns", sizeof("NumberOfColumns")))
	{
		phid->phid.attr.textled.numColumns = value;
	}
	else if(!strncmp(setThing, "Brightness", sizeof("Brightness")))
	{
		phid->brightness = value;
	}
	else{
		ret = EPHIDGET_INVALIDARG;
		DPRINT("Bad setType for TextLED: %s\n", setThing);
	}
	return ret;
}

int phidget_weightsensor_set(CPhidgetHandle generic_phid, const char *setThing, int index, const char *state)
{
	int ret = EPHIDGET_OK;
	CPhidgetWeightSensorHandle phid = (CPhidgetWeightSensorHandle)generic_phid;
	double value = strtod(state, NULL);
	if(!strncmp(setThing, "Weight", sizeof("Weight")))
	{
		phid->Weight = value;
		if (fabs(phid->Weight - phid->lastweight) >= phid->WeightChangeTrigger)
		{
			phid->lastweight = phid->Weight;
			if (phid->fptrWeightChange)
			{
				phid->fptrWeightChange(phid, phid->fptrWeightChangeptr, phid->lastweight);
			}
		}
	}
	else if(!strncmp(setThing, "Trigger", sizeof("Trigger")))
	{
		phid->WeightChangeTrigger = value;
	}
	else{
		ret = EPHIDGET_INVALIDARG;
		DPRINT("Bad setType for WeightSensor: %s\n", setThing);
	}
	return ret;
}

int(*fptrSet[PHIDGET_DEVICE_TYPE_COUNT])(CPhidgetHandle generic_phid, const char *setThing, int index, const char *state) = {NULL, NULL, NULL,
phidget_interfacekit_set,
phidget_servo_set,
NULL, NULL,
phidget_rfid_set,
phidget_weightsensor_set,NULL,
phidget_textlcd_set,
phidget_motorcontrol_set, NULL,
phidget_led_set,
phidget_encoder_set,
phidget_stepper_set,
phidget_temperaturesensor_set,
phidget_accelerometer_set,
phidget_textled_set,
phidget_phsensor_set,
phidget_gyroscope_set,
phidget_advancedservo_set,
phidget_gps_set};

void
network_phidget_event_handler(const char *key, const char *val, pdict_reason_t reason, void *ptr)
{
	CPhidgetHandle phid = (CPhidgetHandle)ptr;
	regmatch_t pmatch[6];
	char *setThing = NULL;
	char *index = NULL;
	char *serial = NULL;
	char errbuf[1024];
	
	int serialNumber;

	int res, ind=-1, version, i, ret = EPHIDGET_OK;

	if(reason!=PDR_ENTRY_REMOVING || !strncmp(val, "Detached", sizeof("Detached")))
	{
		if ((res = regexec(&phidgetsetex, key, 6, pmatch, 0)) != 0) {
			DPRINT("Error in network_phidget_event_handler - pattern not met\n");
			return;
		}
		getmatchsub(key, &serial, pmatch, 2);
		getmatchsub(key, &setThing, pmatch, 3);
		getmatchsub(key, &index, pmatch, 4);

		serialNumber = strtol(serial, NULL, 10);
			
		if(phid->specificDevice == 0 && strncmp(val, "Detached", sizeof("Detached")))
		{
			phid->specificDevice = 2;
			phid->serialNumber = serialNumber;
		}
		
		if(serialNumber == phid->serialNumber && setThing)
		{

			if(!strncmp(setThing, "Label", sizeof("Label")))
			{
				strncpy(phid->label, val, 11);
				phid->keyCount++;
			}
			else if(!strncmp(setThing, "Version", sizeof("Version")))
			{
				version = strtol(val, NULL, 10);
				phid->deviceVersion = version;
				phid->keyCount++;
			}
			else if(!strncmp(setThing, "Name", sizeof("Name")))
			{
				for(i = 0;i<PHIDGET_DEVICE_COUNT;i++)
				{
					if(!strcmp(val, Phid_DeviceSpecificName[i]))
					{
						phid->deviceIDSpec = i;
						phid->attr = Phid_Device_Def[i].pdd_attr;
						phid->keyCount++;
						break;
					}
				}

			}
			else if(!strncmp(setThing, "Status", sizeof("Status")))
			{
				if(!strncmp(val, "Attached", sizeof("Attached")))
				{
					phid->deviceType = Phid_DeviceName[phid->deviceID];
					phid->keyCount++;
					
				}
				else if(!strncmp(val, "Detached", sizeof("Detached")))
				{
					CThread_mutex_lock(&phid->lock);
					phid->keyCount = 0;
					if(phid->specificDevice == 2)
					{
						phid->specificDevice = 0;
					}
					phid->status = PHIDGET_NOTATTACHED;
					CThread_mutex_unlock(&phid->lock);
					if (phid->fptrDetach)
						phid->fptrDetach((CPhidgetHandle)phid, phid->fptrDetachptr);
						
					phid->deviceIDSpec = 0;
					ZEROMEM(&phid->attr, sizeof(CPhidgetAttr));
					ZEROMEM(phid->label, 11);
					phid->deviceVersion = 0;
				}
				else
				{
					throw_error_event(phid, "Bad Message type for Status set", EPHIDGET_NETWORK);
				}
			}
			else if(fptrSet[phid->deviceID] && setThing)
			{
				if(index)
					ind = strtol(index, NULL, 10);
				ret = fptrSet[phid->deviceID](phid, setThing, ind, val);
			}
			
			if((phid->keyCount >= 4) && !CPhidget_isAttached(phid))
			{
				CPhidget_setStatus(phid, PHIDGET_ATTACHED);
				if (phid->fptrAttach)
					phid->fptrAttach(phid, phid->fptrAttachptr);
			}
		}

		free(setThing);
		free(index);
		free(serial);
	}
	
	if(ret)
	{
		snprintf(errbuf, 1024, "Problem during Network set: %s", CPhidget_strerror(ret));
		throw_error_event(phid, errbuf, ret);
	}
}

void
network_manager_event_handler(const char *key, const char *val, pdict_reason_t reason, void *ptr)
{
	CPhidgetManagerHandle phidm = (CPhidgetManagerHandle)ptr;
	regmatch_t pmatch[6];
	char *deviceType = NULL;
	char *serial = NULL;
	char *version = NULL;
	char *deviceIDSpec = NULL;
	char *label = NULL;
	char errbuf[1024];
	
	int serialNumber;
	CPhidgetHandle phid;

	int len, res, ret = EPHIDGET_OK;
	int i;
	
	if(!phidm) return;

	if(reason!=PDR_ENTRY_REMOVING)
	{
		if ((res = regexec(&managerex, key, 6, pmatch, 0)) != 0) {
			DPRINT("Error in network_manager_event_handler - pattern not met\n");
			return;
		}
		getmatchsub(key, &deviceType, pmatch, 1);
		getmatchsub(key, &serial, pmatch, 2);
		getmatchsub(key, &version, pmatch, 4);
		getmatchsub(key, &deviceIDSpec, pmatch, 3);
		getmatchsub(key, &label, pmatch, 5);
		
		serialNumber = strtol(serial, NULL, 10);
	
		if((CPhidget_create(&phid))) return;

		phid->deviceID = phidget_name_to_id(deviceType);
		phid->deviceType = Phid_DeviceName[phid->deviceID];
		phid->serialNumber = serialNumber;
		phid->deviceIDSpec = (unsigned short)strtol(deviceIDSpec, NULL, 10);
		phid->deviceVersion = strtol(version, NULL, 10);
		if(label)
		{
			len = (int)strlen(label);
			if(len>10) len = 10;
			for(i=0;i<len;i++)
			{
				phid->label[i] = label[i];
			}
			phid->label[i] = '\0';
		}

		if(!strcmp(val, "Attached"))
		{
			CPhidget_setStatus(phid, PHIDGET_ATTACHED);
		
			CList_addToList((CListHandle *)phidm->AttachedPhidgets, phid, CPhidget_areEqual);

			if (phidm->fptrAttachChange && phidm->state == PHIDGETMANAGER_ACTIVE)
				phidm->fptrAttachChange((CPhidgetHandle)phid, phidm->fptrAttachChangeptr);
		}
		
		if(!strcmp(val, "Detached"))
		{
			CPhidget_setStatus(phid, PHIDGET_NOTATTACHED);
			if(CList_findInList((CListHandle)phidm->AttachedPhidgets, phid, CPhidget_areEqual, NULL))
			{
				CList_removeFromList((CListHandle *)phidm->AttachedPhidgets, phid, CPhidget_areEqual, PTRUE, CPhidget_free);
			
				if (phidm->fptrDetachChange && phidm->state == PHIDGETMANAGER_ACTIVE)
					phidm->fptrDetachChange((CPhidgetHandle)phid, phidm->fptrDetachChangeptr);
			}
			CPhidget_free(phid);
		}

		free(deviceType);
		free(label);
		free(serial);
		free(version);
		free(deviceIDSpec);
	}
	
	if(ret)
	{
		snprintf(errbuf, 1024, "Problem during Network set: %s", CPhidget_strerror(ret));
		throw_error_event((CPhidgetHandle)phidm, errbuf, ret);
	}
}
