/*
 *  cusblinux.cpp
 *
 *  Filled in by Daniel Risacher on 6/21/05
 *  Filled in more by Patrick McNeil on 6/27/05
 *  Copyright 2005 Phidgets Inc. All rights reserved.
 *
 */

#include "stdafx.h"
#include "cusb.h"



/* CUSBlinux is meant to be platform (in)dependent */

/* This is the "linux" version, but theoretically should work on any
   system with libusb. (linux, OS X, netbsd, freebsd, openbsd, Darwin,
   and windows) But I've only tested it on linux, and not that very well - DRR */

/* So far I've only tested the Accelerometer, and since that only
   reads, the SendPacket code is very suspect.  (2005-06-21) */
/* Tested with the PhidgetServo 3.0 4-motor - so CUSBSendPacket()
   seems okay (2005-06-22) */
/* tested with most Phidgets, also added the PhidgetManager - PM */


int CUSBCloseHandle(CPhidgetHandle phid) {
	if (!phid)
		return EPHIDGET_INVALIDARG;

	CPhidget_setStatus(phid, PHIDGET_NOTATTACHED);
	if (phid->deviceHandle == NULL)
		return EPHIDGET_NOTATTACHED;
		
	usb_release_interface((usb_dev_handle *) phid->deviceHandle, Phid_Device_Def[phid->deviceIDSpec].pdd_iid);
	usb_close((usb_dev_handle *) phid->deviceHandle);
	
	phid->deviceHandle = phid->deviceHandle = NULL;
	
	/*printf("Shut everything down\n");*/
	return EPHIDGET_OK;
}



int CUSBSendPacket(CPhidgetHandle phid, unsigned char *buffer) {
	unsigned long BytesWritten = 0;

	if (!CPhidget_isAttached(phid))
		return EPHIDGET_NOTATTACHED;
		
	BytesWritten = usb_control_msg((usb_dev_handle *)phid->deviceHandle, 
					USB_ENDPOINT_OUT | USB_TYPE_CLASS | USB_RECIP_INTERFACE,
					USB_REQ_SET_CONFIGURATION,
					0x0200, /* value */
					Phid_Device_Def[phid->deviceIDSpec].pdd_iid, /* index*/
					(char *)buffer, 
					phid->outputReportByteLength, /* size */
					1000); /* FIXME? timeout */

	if (BytesWritten == phid->outputReportByteLength) return EPHIDGET_OK;
	else return BytesWritten;
}

int CUSBSetLabel(CPhidgetHandle phid, char *buffer) {
	unsigned long BytesWritten = 0;
	
	if(buffer[0]>22) return EPHIDGET_INVALID;

	if (!CPhidget_isAttached(phid))
		return EPHIDGET_NOTATTACHED;
		
	BytesWritten = usb_control_msg(phid->deviceHandle, 
					USB_ENDPOINT_OUT | USB_TYPE_STANDARD | USB_RECIP_DEVICE,
					USB_REQ_SET_DESCRIPTOR,
					0x0304, /* value */
					0x0409, /* index*/
					(char *)buffer, 
					buffer[0], /* size */
					1000); /* FIXME? timeout */

	if (BytesWritten == buffer[0]) return EPHIDGET_OK;
	else return BytesWritten;
}


/* Buffer should be at least 8 bytes long */
int CUSBReadPacket(CPhidgetHandle phid, unsigned char *buffer) {
	unsigned long BytesRead = 0;
	
	//DPRINT("reading... %d\n",phid->inputReportByteLength);
	
	if (CPhidget_isAttached(phid))
	{
		BytesRead = usb_interrupt_read((usb_dev_handle *)phid->deviceHandle, 
					Phid_Device_Def[phid->deviceIDSpec].pdd_iid+1, 
					(char *)buffer, 
					phid->inputReportByteLength, 
					1000);
		if (phid->inputReportByteLength != BytesRead)
		{
			switch(BytesRead)
			{
				// A timeout occured, but we'll just try again
				case -ETIMEDOUT:
					return EPHIDGET_OK;
				default:
					DPRINT("usb_interrupt_read returned: %lu\n", BytesRead);
					return EPHIDGET_UNEXPECTED;
			}
		}
		return EPHIDGET_OK;
	}
	return EPHIDGET_NOTATTACHED;
}


/* This function needs error checking 
	All this does now is fill in the report sizes - PM
*/
int CUSBGetDeviceCapabilities(CPhidgetHandle phid, struct usb_device *dev, struct usb_dev_handle *udev) {
	/*	FIXME: I'm only assuming this is right. 
		Seems logical and appears to work. DRR */
	//phid->inputReportByteLength = (dev->
	//				       config->
	//				       interface->
	//				       altsetting->
	//				       endpoint->
	//				       wMaxPacketSize);
					       
	/*	This is ugly, but seems to be effective.  
		I stole the design from the mac version.
		And the "fix" for usb_get_descriptor from libhid. DRR 
		Defined as the value after the last 0x95 in the HID descriptor - PM */
	unsigned char buf[255];
	memset(buf, 0, sizeof(buf));
	int len, i;
	len = usb_control_msg(udev, USB_ENDPOINT_IN+1, 
						USB_REQ_GET_DESCRIPTOR,
						(USB_DT_REPORT << 8) + 0, Phid_Device_Def[phid->deviceIDSpec].pdd_iid, (char*)buf, 
						sizeof(buf), 1000 /* ms timeout */);
						
	for(i=10;i<len;i++) {
		if(buf[i]==0x81 && buf[i-2]==0x95)
			phid->inputReportByteLength=buf[i-1];
		if(buf[i]==0x91 && buf[i-2]==0x95)
			phid->outputReportByteLength=buf[i-1];
	}					
	
	len = usb_get_string_simple(udev, 4, (char *)phid->label, 11);

	
	return EPHIDGET_OK;
}


/* What should this do? It's only called once from
   cphidgetmanager.cpp, and it's stubbed out in the mac version, so
   maybe it's not important - DRR

   This needs to maintain a list of devices - it's used by the phidget manager for
   keeping track of attach and detach events. (On Mac this is handled with 
   notifications, so this is not needed.) - PM
*/

int CUSBBuildList(CPhidgetList **curList) {
	int MemberIndex, i, found, ret;
	unsigned long Length;
	CPhidgetList *traverse;
	Length = 0;
	MemberIndex = 0;
	struct usb_bus *bus;
	struct usb_device *dev;
	usb_dev_handle *udev;
	CPhidgetHandle phid;
	
	unsigned char *fhandle = 0;

	TESTPTR(curList)
	
	usb_init();
	//printf("got here 1.0\n");
	ret = usb_find_busses();
	//printf("got here 1.1\n");
	ret = usb_find_devices();
	
	//search through all USB devices
	for (bus = usb_busses; bus; bus = bus->next) {
		for (dev = bus->devices; dev; dev = dev->next) {
		
			found = 0;
			if (AttachedDevices) {
				for (traverse = AttachedDevices; traverse;
					traverse=traverse->next) {
					fhandle = traverse->phid->CPhidgetFHandle;
					if (!strncmp(fhandle, dev->filename, 3)) {
						/*if (!(phid = (CPhidgetHandle)malloc(sizeof (*phid))))
							return EPHIDGET_NOMEMORY;
						ZEROMEM(phid, sizeof(*phid));
						memcpy(phid, traverse->phid,
							sizeof(*phid));
						AddDeviceToList(curList, phid);*/
						CList_addToList((CListHandle *)curList, traverse->phid, CPhidget_areEqual);
						found = 1;
					}
				}
			}
			if (!found) {
				for (i = 1; i<PHIDGET_DEVICE_COUNT; i++) {	 
					if ((dev->descriptor.idVendor == Phid_Device_Def[i].pdd_vid) && 
						(dev->descriptor.idProduct == Phid_Device_Def[i].pdd_pid)) {
						if (!(phid = (CPhidgetHandle)malloc(
						    sizeof (*phid))))
							return EPHIDGET_NOMEMORY;
						ZEROMEM(phid, sizeof(*phid));
						
						udev = usb_open(dev);
						if(udev) {
// No real need for this in the buildlist
//#ifdef LIBUSB_HAS_DETACH_KERNEL_DRIVER_NP
						  //ret = usb_detach_kernel_driver_np(udev, Phid_Device_Def[phid->deviceIDSpec].pdd_iid);
//#endif
							if (dev->descriptor.bcdDevice < 0x100) 
								phid->deviceVersion = dev->descriptor.bcdDevice * 100;
							else
								phid->deviceVersion = ((dev->descriptor.bcdDevice >> 8) * 100) + ((dev->descriptor.bcdDevice & 0xff));
							phid->deviceType = (char *)Phid_DeviceName[Phid_Device_Def[i].pdd_did];

						
							phid->status = PHIDGET_ATTACHED;
							phid->deviceIDSpec = i;
							phid->deviceID = Phid_Device_Def[i].pdd_did;
							phid->ProductID = dev->descriptor.idProduct;
							phid->VendorID = dev->descriptor.idVendor;
							//phid->deviceHandle = (long) udev;
							
							if (dev->descriptor.iSerialNumber) {
								char string[256];
								memset(string, 0, 256);
								usb_get_string_simple(udev, dev->descriptor.iSerialNumber, string, sizeof(string));
								//printf("Found Serial: %s, %d, %d\n",string,dev->descriptor.iSerialNumber, ret);
								if(atol(string)!=0)
									phid->serialNumber = atol(string);
								else
									phid->serialNumber = -1;
							}
							phid->specificDevice = TRUE;
							phid->attr = Phid_Device_Def[i].pdd_attr;
							
							// This is not needed for manager devices!!
							/*if(!usb_claim_interface(udev, Phid_Device_Def[phid->deviceIDSpec].pdd_iid)) {
								CUSBGetDeviceCapabilities(phid, dev, udev);
							}
							usb_release_interface(udev, Phid_Device_Def[phid->deviceIDSpec].pdd_iid);*/
							
							phid->CPhidgetFHandle = (char *)malloc(4);
							fhandle = phid->CPhidgetFHandle;
							strncpy(fhandle, dev->filename, 3);
							fhandle[3] = '\0';
							
							DPRINT("new device found: %s\n", fhandle);

							usb_close(udev);
							CList_addToList((CListHandle *)curList, phid, CPhidget_areEqual);
						}
						else
						{
							usb_close(udev);
						}
					}
				}
      		}
      	}
    }
    return EPHIDGET_OK;
}

void CUSBCleanup(void)
{
	usb_cleanup();
}
/* 
	CUSBOpenHandle takes a CPhidgetInfo structure, with 
	ProductID/VendorID/SerialNumber filled in.

	If SerialNumber is = -1, CUSBOpenHandle will return with the first available
	HID Phidget that matches the ProductID/VendorID.

	CUSBOpenHandle should reserve the file handles - in other words,
	a future call to CUSBOpenHandle should fail.
*/


/* Based on analysis of the Windows code, I think the comment above is
   WRONG. This should assume that deviceID is filled in, but not that
   the ProductID/VendorID are. DRR */

int CUSBOpenHandle(CPhidgetHandle phid)
{
	int idVendor;
	int idProduct;
	int i,ret;
	struct usb_bus *bus;
	struct usb_device *dev;
	usb_dev_handle *udev;
	usb_init();
	//printf("got here 2.0\n");
	ret = usb_find_busses();
	//printf("got here 2.1\n");
	ret = usb_find_devices();

	for (bus = usb_busses; bus; bus = bus->next) {
		for (dev = bus->devices; dev; dev = dev->next) {
			for (i = 1; i<PHIDGET_DEVICE_COUNT; i++) {
				if (Phid_Device_Def[i].pdd_did == phid->deviceID) {
					idVendor = Phid_Device_Def[i].pdd_vid;
					idProduct = Phid_Device_Def[i].pdd_pid;
					if ((dev->descriptor.idVendor == idVendor) && (dev->descriptor.idProduct == idProduct)) {
						/* the vend/prod matches! */
						udev = usb_open(dev);
						if (udev) {
							/*	On Linux, the HID driver likes to claim Phidgets - we can disconnect it here.
								Note this is Linux Only - PM */	
#ifdef LIBUSB_HAS_DETACH_KERNEL_DRIVER_NP
							ret = usb_detach_kernel_driver_np(udev, Phid_Device_Def[i].pdd_iid);
#endif
							if(!usb_claim_interface(udev, Phid_Device_Def[i].pdd_iid)) {
								long serial;
								if (dev->descriptor.iSerialNumber) {
									char string[256];
									ret = usb_get_string_simple(udev, dev->descriptor.iSerialNumber, string, sizeof(string));
									serial = atol(string);
								}
								if (phid->specificDevice == FALSE || serial == phid->serialNumber) {
									/* the serialnum is okay */
									
									//CPhidget_setStatus(phid, PHIDGET_ATTACHED);

									phid->deviceHandle = (HANDLE)udev;
									phid->deviceIDSpec = i;
									phid->deviceType = (char *)Phid_DeviceName[Phid_Device_Def[i].pdd_did];
			
									phid->ProductID = idProduct;
									phid->VendorID = idVendor;
									if (dev->descriptor.bcdDevice < 0x100)
										phid->deviceVersion = dev->descriptor.bcdDevice * 100;
									else
										phid->deviceVersion = ((dev->descriptor.bcdDevice >> 8) * 100) + ((dev->descriptor.bcdDevice & 0xff));			
									phid->serialNumber = serial;
			
									CUSBGetDeviceCapabilities(phid, dev, udev);
	
									phid->attr = Phid_Device_Def[i].pdd_attr;
			
									if ((Phid_Device_Def_WorkAroundA[i][0]) && (Phid_Device_Def_WorkAroundA[i][1] >= dev->descriptor.bcdDevice)) {
										unsigned char buffer[8];
										ZEROMEM(buffer,8);
										CUSBSendPacket((CPhidgetHandle)phid, buffer);
									}

									return EPHIDGET_OK;
								}
								else
								{
									usb_release_interface(udev, Phid_Device_Def[i].pdd_iid);
									usb_close(udev);
								}
	      					} /* serial number */
	      					else
							{
								usb_close(udev);
							}
	    				} /* udev open */
	  				} /* vendor/product match */
				} /* deviceID matches in table */
      		} /* iterate over phidget device table */
    	} /* iterate over USB devices */
  	} /* iterate over USB busses */
  	return EPHIDGET_NOTFOUND;
}
