#include "stdafx.h"
#include <stdio.h>
#if !defined(_WINDOWS) && !defined(_CYGWIN)
#include <sys/socket.h>
#include <sys/uio.h>
#include <pthread.h>
#else
#include "wincompat.h"
#endif
#ifndef _MSC_EXTENSIONS
#include <unistd.h>
#include <signal.h>
#endif
#include <string.h>
#include "pdictserver.h"
#include "utils.h"

#define WRITE(fd, msg) send(fd, msg, strlen(msg), 0)

pdict_t *pd;
pthread_mutex_t pd_mutex;
pds_session_t *last_pdss = 0;

static void
pd_lock(void *pd_lock) {
	(void) pthread_mutex_lock(pd_lock);
}

static void
pd_unlock(void *pd_lock) {
	(void) pthread_mutex_unlock(pd_lock);
}

static void
accept_cb(int fd, const char *claddr, int clport)
{
	const pds_session_t *pdss;
	char errdesc[1024];
	pthread_t t;
#ifndef _MSC_EXTENSIONS
	sigset_t new, old;
#endif
	char *buf;

#ifndef _MSC_EXTENSIONS
	sigemptyset(&new);
	sigaddset(&new, SIGPIPE);
	sigprocmask(SIG_BLOCK, &new, NULL);
#endif
	if (!(pdss = pds_session_alloc(pd, pd_lock, pd_unlock, &pd_mutex, fd,
	    pu_read, fd, pu_write, pu_close, pu_log, NULL, errdesc,
	    sizeof (errdesc)))) {
		pu_log(PUL_ERR, 0, errdesc);
		perror("pds_session_alloc");
		(void) WRITE(fd, "500 temporarily unavailable\n");
		pu_close(fd, errdesc, sizeof (errdesc));
		return;
	}
#ifndef _MSC_EXTENSIONS
	sigfillset(&new);
	pthread_sigmask(SIG_BLOCK, &new, &old);
#endif
	if (pthread_create(&t, NULL, (void *(*)(void *))pds_session_serve,
	    (void *)pdss) != 0) {
		pu_log(PUL_ERR, 0, errdesc);
		perror("pds_session_alloc");
		(void) WRITE(fd, "500 temporarily unavailable\n");
		pu_close(fd, errdesc, sizeof (errdesc));
		return;
	}
#ifndef _MSC_EXTENSIONS
	pthread_sigmask(SIG_SETMASK, &old, NULL);
#endif
	pasprintf(&buf, "accepted session from %s:%d", claddr, clport);
	if (buf) {
		pu_log(PUL_INFO, pds_session_id(pdss), buf);
		free(buf);
	}
	last_pdss = (pds_session_t *)pdss;
}

void *
setandflush(void *arg)
{
	static int nchange = 0;
	char buf[20];

	for (;;) {
		if (!last_pdss)
			continue;
		snprintf(buf, sizeof (buf), "%d", ++nchange);
		pd_lock(&pd_mutex);
		pdict_add(pd, "nchange", buf, NULL);
		pd_unlock(&pd_mutex);
		pds_session_flush(last_pdss);
		sleep(1);
	}
}

int
main(void)
{
	char errdesc[1024];

#ifdef _WINDOWS
	{
		WSADATA wsad;
		WORD wsav;

		wsav = MAKEWORD(2, 2);
		WSAStartup(wsav, &wsad);
	}
#endif
	if (!(pd = pdict_alloc())) {
		perror("pdict_alloc");
		return 1;
	}
	if (pthread_mutex_init(&pd_mutex, NULL) != 0) {
		perror("pthread_mutex_init");
		return 1;
	}

	if (!pds_init()) {
		perror("pds_init");
		return 1;
	}
#ifndef _MSC_EXTENSIONS
#ifdef SERVER_NODETACH
	if (!(pdss = pds_session_alloc(pd, pd_lock, pd_unlock, &pd_mutex, 0,
	    pu_read, 1, pu_write, 0, errdesc, sizeof (errdesc)))) {
		perror("pds_session_alloc");
		return 1;
	}
	pthread_create(&t, NULL, (void *(*)(void *))pds_session_serve,
	    (void *)pdss);
	/* XXX pdss leaked */
#endif
#endif
	if (0) {
		pthread_t f;
		pthread_create(&f, NULL, setandflush, NULL);
	}
	if (!stream_server_accept(5001, accept_cb, errdesc, sizeof (errdesc))) {
		fprintf(stderr, "stream_server_accept: %s\n", errdesc);
		return 1;
	}
	return 0;
}
