#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <assert.h>
#include "ptree.h"

ptree_node_t *root = NULL;

int
compare_int(const void *v1, const void *v2)
{
	return (int)v1 - (int)v2;
}

ptree_walk_res_t
print_int(const void *v1, int level, void *arg,
    void *ptree_inorder_walking_remove_arg)
{
	while (level--)
		printf("  ");
	printf("%d\n", (int)v1);
	return PTREE_WALK_CONTINUE;
}

ptree_walk_res_t
print_and_remove_int(const void *v1, int level, void *arg,
    void *pwra)
{
	void *ov;
	while (level--)
		printf("  ");
	printf("%d", (int)v1);
	if ((int)arg == (int)v1) {
		printf(" (removing)\n");
		ptree_inorder_walking_remove(&root, &ov, pwra);
		assert((int)ov == (int)v1);
	} else
		printf("\n");
	return PTREE_WALK_CONTINUE;
}

int
main(void)
{
	void *oldv = NULL;

	assert(ptree_replace((void *)5, &root, compare_int, &oldv));
	assert(!oldv);
	assert(ptree_replace((void *)3, &root, compare_int, &oldv));
	assert(!oldv);
	assert(ptree_replace((void *)7, &root, compare_int, &oldv));
	assert(!oldv);
	assert(ptree_replace((void *)7, &root, compare_int, &oldv));
	assert((int)oldv == 7);
	assert(ptree_contains((void *)7, root, compare_int, NULL));
	assert(!ptree_contains((void *)4, root, compare_int, NULL));
	printf("inorder:\n");
	ptree_walk(root, PTREE_INORDER, print_int, 0);
	printf("preorder:\n");
	ptree_walk(root, PTREE_PREORDER, print_int, 0);
	printf("postorder:\n");
	ptree_walk(root, PTREE_POSTORDER, print_int, 0);
	assert(root);
	ptree_clear(&root);
	assert(!root);
	printf("after clear, inorder:\n");
	ptree_walk(root, PTREE_INORDER, print_int, 0);

	/* removal */
	assert(ptree_replace((void *)5, &root, compare_int, &oldv));
	assert(ptree_replace((void *)3, &root, compare_int, &oldv));
	assert(ptree_replace((void *)7, &root, compare_int, &oldv));

	assert(!ptree_remove((void *)6, &root, compare_int, &oldv));

	assert(ptree_remove((void *)3, &root, compare_int, &oldv));
	assert((int)oldv == 3);
	printf("after remove (3) from ((3) 5 (7)):\n");
	ptree_walk(root, PTREE_INORDER, print_int, 0);

	assert(ptree_replace((void *)3, &root, compare_int, &oldv));
	assert(ptree_remove((void *)5, &root, compare_int, &oldv));
	assert((int)oldv == 5);
	printf("after remove (5) from ((3) 5 (7)):\n");
	ptree_walk(root, PTREE_INORDER, print_int, 0);

	/* in-walk removal */
	assert(ptree_replace((void *)5, &root, compare_int, &oldv));
	assert(ptree_replace((void *)3, &root, compare_int, &oldv));
	assert(ptree_replace((void *)7, &root, compare_int, &oldv));
	printf("during in-walk removal of (5) from ((3) 5 (7)):\n");
	ptree_walk(root, PTREE_INORDER, print_and_remove_int, (void *)5);
	printf("after in-walk removal of (5) from ((3) 5 (7)):\n");
	ptree_walk(root, PTREE_INORDER, print_int, 0);
	assert(ptree_contains((void *)3, root, compare_int, 0));
	assert(!ptree_contains((void *)5, root, compare_int, 0));
	assert(ptree_contains((void *)7, root, compare_int, 0));
}
