#include "stdafx.h"
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include "wincompat.h"

#ifdef _MSC_EXTENSIONS
#define EAGAIN          11
#define ENOMEM          12
#define ENOSYS          40
#endif

struct pthread {
	void *pt_handle;
	unsigned long pt_id;
};

struct pthread_cond {
	HANDLE ptc_handle;
};

struct pthread_mutex {
	CRITICAL_SECTION ptm_cs;
};

pthread_t pthread_self(void)
{
	return (pthread_t)GetCurrentThreadId();
}

int
pthread_create(pthread_t *pt, const pthread_attr_t *attr,
    void *(*start)(void *), void *arg)
{
	if (!(*pt = malloc(sizeof (**pt))))
		return errno = ENOMEM;
	(*pt)->pt_handle = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)start,
	    arg, 0, &(*pt)->pt_id);
	if (!(*pt)->pt_id)
		return errno = EAGAIN;

	return 0;
}

int
pthread_join(pthread_t pt, void **status)
{
	DWORD ec;

	while (GetExitCodeThread(pt->pt_handle, &ec) && ec == STILL_ACTIVE)
		Sleep(100);
	if (status)
		*(DWORD *)status = ec;
	return 0;
}

int
pthread_mutex_init(pthread_mutex_t *ptm, const pthread_mutexattr_t *attr)
{
	if (attr)
		return errno = ENOSYS;
	if (!(*ptm = malloc(sizeof (**ptm))))
		return errno = ENOMEM;
	InitializeCriticalSection(&(*ptm)->ptm_cs);
	return 0;
}

int
pthread_mutex_destroy(pthread_mutex_t *ptm)
{
	DeleteCriticalSection(&(*ptm)->ptm_cs);
	free(*ptm);
	return 0;
}

int
pthread_mutex_lock(pthread_mutex_t *ptm)
{
	EnterCriticalSection(&(*ptm)->ptm_cs);
	return 0;
}

int
pthread_mutex_unlock(pthread_mutex_t *ptm)
{
	LeaveCriticalSection(&(*ptm)->ptm_cs);
	return 0;
}

int
pthread_cond_init(pthread_cond_t *cond, const pthread_condattr_t *attr)
{
	if (attr)
		return errno = ENOSYS;
	if (!(*cond = malloc(sizeof (**cond))))
		return errno = ENOMEM;
		
	(*cond)->ptc_handle = CreateEvent(NULL, FALSE, FALSE, NULL);
	return 0;
}

int
pthread_cond_destroy(pthread_cond_t *cond)
{
	CloseHandle((*cond)->ptc_handle);
	free(*cond);
	return 0;
}

int
pthread_cond_wait(pthread_cond_t *cond, pthread_mutex_t *mutex)
{
	int res;

	pthread_mutex_unlock(mutex);
	res = WaitForSingleObject((*cond)->ptc_handle, INFINITE);
	pthread_mutex_lock(mutex);
	if (res != WAIT_OBJECT_0)
		return errno = ENOSYS;
	return 0;
}

int
pthread_cond_signal(pthread_cond_t *ptc)
{
	ResetEvent((*ptc)->ptc_handle);
	SetEvent((*ptc)->ptc_handle);
	return 0;
}

int
usleep(unsigned long useconds)
{
	Sleep(useconds / 1000);
	return 0;
}

void
wsa_set_errdesc(char *errdesc, int len)
{
	int err;

	if (!errdesc)
		return;
	switch (err = WSAGetLastError()) {
	case 10038:
		snprintf(errdesc, len, "Socket operation on non-socket");
		return;
	case 10061:
		snprintf(errdesc, len, "Connection refused");
		return;
	default:
		snprintf(errdesc, len, "WSA error %d", err);
		return;
	}
}


#if (defined(_WINDOWS) && defined(__GNUC__)) || defined(WINCE)
char *
strdup(const char *s)
{
	char *d = NULL;
	int l;

	if (!(d = malloc(l = (strlen(s)+1))))
		return NULL;
	memcpy(d, s, l);
	return d;
}
#endif
